#include "GameCore.h"

CGameCore::CGameCore()
{
	graphics=NULL;
	input=NULL;
//	sound=NULL;
	ShutdownBool=false;
	CurrentFrameRate=0;
}

CGameCore::~CGameCore()
{
	graphics=NULL;
	input=NULL;
//	sound=NULL;
	ShutdownBool=false;
	CurrentFrameRate=0;
}

void CGameCore::HandleCoreEvents()
{
	if(input->LastEvent.type==SDL_VIDEORESIZE)
	{
		graphics->ResizeWindow(input->LastEvent.resize.w, input->LastEvent.resize.h);
	}

	if(input->LastEvent.type==SDL_QUIT)
		Shutdown();
}

bool CGameCore::Init(InitOptions Options)
{
	graphics = new CGraphicsSystem();
	input = new CInputSystem();

	if(!graphics->Init(Options.Width, Options.Height, Options.Bpp, Options.Depth, (char *)Options.Title, (char *)Options.Icon, Options.Fullscreen, Options.Audio))
		return false;

	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);
	ilEnable(IL_ORIGIN_SET);
	ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
	
	return true;
}

int CGameCore::Run()
{
	float dt;
	float CurrentTime=SDL_GetTicks();
	static float LastTime=CurrentTime;
	static float LastFrameRateTime=CurrentTime;
	static int NumFramesAdded=0;

	if(!Initialize())
	{
		Cleanup();
		if(graphics)
			delete graphics;
		if(input)
			delete input;

		SDL_Quit();
		return 0;
	}

	while(!ShutdownBool)
	{
		CurrentTime=SDL_GetTicks();
		dt=CurrentTime-LastTime;
		LastTime=CurrentTime;

		while(input->UpdateEvents())
		{
			HandleUserEvents(dt);			//handle all in-game events
			HandleCoreEvents();				//handle the window events and any other non-game event
		}

		if(!Update(dt))						//Update the Frame so we could render
			Shutdown();
		

		graphics->BeginRender();
			Render();						// Render the scene
		graphics->EndRender();
		float CurrentFrameRateTime=SDL_GetTicks();

		NumFramesAdded++;

		if((CurrentFrameRateTime-LastFrameRateTime)>1000.0f)
		{
			CurrentFrameRate=NumFramesAdded/((CurrentFrameRateTime-LastFrameRateTime)*0.001f);
			LastFrameRateTime=CurrentFrameRateTime;
			NumFramesAdded=0;
		}

	}

	Cleanup();
	if(graphics)
		delete graphics;
	if(input)
		delete input;
//	if(audio)
//		delete audio;

	SDL_Quit();
	return 0;
}

