#include "includes.h"
#include "CGraphicsSystem.h"

CGraphicsSystem::CGraphicsSystem()
{
	Width=800;
	Height=600;				
	Window=NULL;

	FieldOfView=45.0f;
	DepthBits=16;
	BitsPerPixel=16;
	Fullscreen=true;
}

CGraphicsSystem::~CGraphicsSystem()
{
	SDL_FreeSurface(Window);

	Width=0;
	Height=0;				
	Window=NULL;

	FieldOfView=0.0f;
	DepthBits=0;
	BitsPerPixel=0;
	Fullscreen=false;
}

bool CGraphicsSystem::Init(int width, int height, int bpp, int depth, char * Title, char * Icon, bool fullscreen, bool audio)
{
	if(SDL_InitSubSystem(SDL_INIT_VIDEO) < 0)                      // try to initialize SDL video module
	{
		return false;
	}
	
	Width=width;
	Height=height;
	BitsPerPixel=bpp;
	DepthBits=depth;
	Fullscreen=fullscreen;
	WindowTitle=(char *)Title;
	WindowIcon=(char *)Icon;

	VideoFlags   = SDL_OPENGL;                         // it's an openGL window
	VideoFlags   |= SDL_HWPALETTE;                      // exclusive access to hardware colour palette

	if(fullscreen)
		VideoFlags |=SDL_FULLSCREEN;

	const SDL_VideoInfo * VideoInfo = SDL_GetVideoInfo();     // query SDL for information about our video hardware

	if(VideoInfo == NULL)                                     // if this query fails
	{
		return false;
	}

	///////////// we set the system dependant flags here
	if(VideoInfo -> hw_available)                      // is it a hardware surface
		VideoFlags |= SDL_HWSURFACE;
	else
		VideoFlags |= SDL_SWSURFACE;

	if(VideoInfo -> blit_hw)                            // is hardware blitting available
		VideoFlags |= SDL_HWACCEL;


	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );      //OpenGL Double Buffered
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, depth);         // set size of depth buffer

	Window = SDL_SetVideoMode(width, height, bpp, VideoFlags);     // SCREEN_DEPTH is macro for bits per pixel

	if( Window == NULL )                            // if window creation failed
	{
		return false;
	}


	SDL_WM_SetCaption(Title, Title);  // set the window caption

	ResizeViewport(0, 0, width, height);

	return true;
}

bool CGraphicsSystem::ResizeWindow(int width, int height)
{
    if(height==0)                                        // Prevent A Divide By Zero error
    {
        height=1;                                        // Make the Height Equal One
    }

	Width=width;
	Height=height;

	// request SDL to resize the window to the size and depth etc. that we specified
	Window = SDL_SetVideoMode(Width, Height, BitsPerPixel, VideoFlags);

	if(Window == NULL)                              // if window resize has failed
	{
		return false;
	}

	return true;
}

bool CGraphicsSystem::ResizeViewport(int X, int Y, int width, int height)
{
    glViewport(X,Y,width,height);						//update the viewport

    glMatrixMode(GL_PROJECTION);                        // Select The Projection Matrix
    glLoadIdentity();                                   // Reset The Projection Matrix

	glOrtho(0.0f, width-1.0, 0.0, height-1.0, -1.0, 1.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	return true;
}

void CGraphicsSystem::BeginRender()
{
    glMatrixMode(GL_PROJECTION);                        // Select The Projection Matrix
    glLoadIdentity();                                   // Reset The Projection Matrix

	glOrtho(0.0f, Width-1.0, 0.0, Height-1.0, -1.0, 1.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear The Appropriate Buffers
}

void CGraphicsSystem::EndRender()
{
	SDL_GL_SwapBuffers();					// Swap display buffers
}

