#include "Face.h"

FaceTexture::FaceTexture()
{
	AniFrames.clear();
	CurrentTexture=0;
	FramesPerSec=0;
}

FaceTexture::~FaceTexture()
{
	AniFrames.clear();
	CurrentTexture=0;
	FramesPerSec=0;
}

bool FaceTexture::FrameAdd(char *filename)
{
	AniFrames.push_back(CImage());
	if((AniFrames[AniFrames.size()-1].LoadImage(filename))==false)
	{
		AniFrames.pop_back();
		return false;
	}
	
	return true;
}

bool FaceTexture::FrameAddWAlpha(char *filename, int AR, int AG, int AB)
{
	AniFrames.push_back(CImage());
	if((AniFrames[AniFrames.size()-1].LoadImageWAlpha(filename, AR, AG, AB))==false)
	{
		AniFrames.pop_back();
		return false;
	}
	
	return true;
}

bool FaceTexture::FrameAddXYWH(char *filename, int x, int y, int w, int h)
{
	AniFrames.push_back(CImage());
	if((AniFrames[AniFrames.size()-1].LoadImageXYWH(filename, x, y, w, h, false))==false)
	{
		AniFrames.pop_back();
		return false;
	}
	
	return true;
}

bool FaceTexture::UpdateTex(float dt)
{
	if(AniFrames.size()<2)
		CurrentTexture=0;
	else
	{
		static int CurrentFrame=0;
		static int CurrentTimeToPass=1;
		float CurrentTime=SDL_GetTicks();
		static float LastTime=CurrentTime;
		float FrameMultiplier=1000.0f/FramesPerSec;
		static int LastFrameRate=FramesPerSec;
		if(LastFrameRate!=FramesPerSec)
		{
			CurrentTimeToPass=1;
			LastTime=CurrentTime;
			LastFrameRate=FramesPerSec;
		}

		float ElapsedTime=CurrentTime-LastTime;
		
		if(ElapsedTime>=(FrameMultiplier*CurrentTimeToPass))
		{
			CurrentTimeToPass+=1;
			CurrentFrame+=1;
			if(ElapsedTime>=1000.0f)
				LastTime=CurrentTime;
		}

		if(CurrentTimeToPass>FramesPerSec)
			CurrentTimeToPass=1;

		if(CurrentFrame>(AniFrames.size()-1))
			CurrentFrame=0;
		
		CurrentTexture=CurrentFrame;
	}
	return true;
}

void FaceTexture::BindTexture()
{
	AniFrames[CurrentTexture].BindTexture();
}

void FaceTexture::Cleanup()
{
	for(int i=0; i<AniFrames.size(); i++)
		AniFrames[i].DestroyTexture();
}
	
	