#include "ImageSystem.h"

CImage::CImage()
{
	TextureName=-1;
	TextureCreated=false;
}

CImage::~CImage()
{
}

bool CImage::LoadImage(char *ImageName)
{
	unsigned int ImageID;

	ilGenImages(1, &ImageID);

	ilBindImage(ImageID);
	if(IL_FALSE==ilLoadImage(ImageName))
	{
		ILenum Error;
		while((Error=ilGetError())!=IL_NO_ERROR)
		{
			MessageBox(NULL, iluErrorString(Error), "CImage Load Error:", MB_OK | MB_ICONERROR);
		}
		return false;
	}
	else
	{
		int Bpp=ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);

		if(Bpp==4)
			ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);
		else
			ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);

		// Generate a texture with the associative texture ID stored in the array
		glGenTextures(1, &TextureName);

		// Bind the texture
		glBindTexture(GL_TEXTURE_2D, TextureName);

		// This sets the alignment requirements for the start of each pixel row in memory.
		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);

		if(Bpp==4)
		{
			//Build Mipmaps with an Alpha channel too
			gluBuild2DMipmaps(GL_TEXTURE_2D, 4, ilGetInteger(IL_IMAGE_WIDTH), ilGetInteger(IL_IMAGE_HEIGHT), GL_RGBA, GL_UNSIGNED_BYTE, ilGetData());
		}
		else 
		{
			//Build Mipmaps
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, ilGetInteger(IL_IMAGE_WIDTH), ilGetInteger(IL_IMAGE_HEIGHT), GL_RGB, GL_UNSIGNED_BYTE, ilGetData());
		}
	}
	
	if(ilIsImage(ImageID))
		ilDeleteImages(1, &ImageID);

	TextureCreated=true;
	return true;
}

bool CImage::LoadImageWAlpha(char *ImageName, int AR, int AG, int AB)		//Loads the image with an alpha channel by removing the selected color and then turns it into a texture
{
	unsigned int ImageID;

	ilGenImages(1, &ImageID);

	ilBindImage(ImageID);
	if(IL_FALSE==ilLoadImage(ImageName))
	{
		ILenum Error;
		while((Error=ilGetError())!=IL_NO_ERROR)
		{
			MessageBox(NULL, iluErrorString(Error), "CImage Load Error:", MB_OK | MB_ICONERROR);
		}
		return false;
	}
	else
	{
		int Bpp=ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
		int ImageWidth=ilGetInteger(IL_IMAGE_WIDTH);
		int ImageHeight=ilGetInteger(IL_IMAGE_HEIGHT);
		
		Bpp=4;
		ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);

		unsigned char *TempBuffer=(unsigned char*)ilGetData();
		unsigned char *Buffer=(unsigned char*)malloc(ImageWidth*ImageHeight*Bpp);
	
		int position=0;
		for (int i=0; i<ImageHeight; i++)
		{
			 for (int c=0; c<ImageWidth; c++)
			 {
				 Buffer[position]   = TempBuffer[(c*Bpp) + ((i * ImageWidth) * Bpp)];
				 Buffer[position+1] = TempBuffer[(c*Bpp+1) + ((i * ImageWidth) * Bpp)];
				 Buffer[position+2] = TempBuffer[(c*Bpp+2) + ((i * ImageWidth) * Bpp)];
				 if(Buffer[position]==AR && Buffer[position+1]==AG && Buffer[position+2]==AB)
					 Buffer[position+3] = 0x00;
				 else
					 Buffer[position+3] = 0xFF;

				 position+=4;
			 }
		}


		// Generate a texture with the associative texture ID stored in the array
		glGenTextures(1, &TextureName);

		// Bind the texture
		glBindTexture(GL_TEXTURE_2D, TextureName);

		// This sets the alignment requirements for the start of each pixel row in memory.
		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);

		//Build Mipmaps with an Alpha channel too
		gluBuild2DMipmaps(GL_TEXTURE_2D, 4, ImageWidth, ImageHeight, GL_RGBA, GL_UNSIGNED_BYTE, (unsigned char*)Buffer);

		free(Buffer);
	}
	
	if(ilIsImage(ImageID))
		ilDeleteImages(1, &ImageID);

	TextureCreated=true;
	return true;
}

void CImage::BindTexture()
{
	if(TextureCreated)
		glBindTexture(GL_TEXTURE_2D, TextureName);
}

unsigned int CImage::GetTextureToBind()
{
	if(TextureCreated)
		return (TextureName);

	return -1;
}

void CImage::DestroyTexture()
{
	if(TextureCreated)
		if(glIsTexture(TextureName))
			glDeleteTextures(1, &TextureName);
}

bool CImage::LoadImageXYWH(char *ImageName, int x, int y, int w, int h, bool LLorigin)
{
	unsigned int ImageID;

	ilGenImages(1, &ImageID);

	ilBindImage(ImageID);
	if(IL_FALSE==ilLoadImage(ImageName))
	{
		ILenum Error;
		while((Error=ilGetError())!=IL_NO_ERROR)
		{
			MessageBox(NULL, iluErrorString(Error), "CImage Load Error:", MB_OK | MB_ICONERROR);
		}
		return false;
	}
	else
	{
		int Bpp=ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
		int ImageWidth=ilGetInteger(IL_IMAGE_WIDTH);
		int ImageHeight=ilGetInteger(IL_IMAGE_HEIGHT);
		
		if(!LLorigin)
			y=ImageHeight-(y+h);

		if(Bpp==4)
			ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);
		else
		{
			Bpp=3;
			ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
		}

		if(w>(ilGetInteger(IL_IMAGE_WIDTH)))
			w=(ilGetInteger(IL_IMAGE_WIDTH));
		if(h>(ilGetInteger(IL_IMAGE_HEIGHT)))
			h=(ilGetInteger(IL_IMAGE_HEIGHT));

		unsigned char *TempBuffer=(unsigned char*)ilGetData();
		unsigned char *Buffer=(unsigned char*)malloc(w*h*Bpp);
	
		int position=0;
		if(Bpp==4)
		{
			for (int i=y; i<(y+h); i++)
			{
				 for (int c=x; c<(x+w); c++)
				 {
					 Buffer[position]   = TempBuffer[(c*Bpp) + ((i * ImageWidth) * Bpp)];
					 Buffer[position+1] = TempBuffer[(c*Bpp+1) + ((i * ImageWidth) * Bpp)];
					 Buffer[position+2] = TempBuffer[(c*Bpp+2) + ((i * ImageWidth) * Bpp)];
					 Buffer[position+3] = TempBuffer[(c*Bpp+3) + ((i * ImageWidth) * Bpp)];
					 position+=4;
				 }
			}
		}
		else
		{
			for (int i=y; i<(y+h); i++)
			{
				 for (int c=x; c<(x+w); c++)
				 {
					 Buffer[position]   = TempBuffer[(c*Bpp) + ((i * ImageWidth) * Bpp)];
					 Buffer[position+1] = TempBuffer[(c*Bpp+1) + ((i * ImageWidth) * Bpp)];
					 Buffer[position+2] = TempBuffer[(c*Bpp+2) + ((i * ImageWidth) * Bpp)];
					 position+=3;
				 }
			}
		}

		// Generate a texture with the associative texture ID stored in the array
		glGenTextures(1, &TextureName);

		// Bind the texture
		glBindTexture(GL_TEXTURE_2D, TextureName);

		// This sets the alignment requirements for the start of each pixel row in memory.
		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);

		if(Bpp==4)
		{
			//Build Mipmaps with an Alpha channel too
			gluBuild2DMipmaps(GL_TEXTURE_2D, 4, w, h, GL_RGBA, GL_UNSIGNED_BYTE, (unsigned char*)Buffer);
		}
		else 
		{
			//Build Mipmaps
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, w, h, GL_RGB, GL_UNSIGNED_BYTE, (unsigned char*)Buffer);
		}
		free(Buffer);
	}
	
	if(ilIsImage(ImageID))
		ilDeleteImages(1, &ImageID);

	TextureCreated=true;
	return true;
}