#include "collection.h"

#include <vmgp.h>
#include <vstream.h>

#include "globals.h"
#include "res.h"
#include "app.h"
#include "utility.h"

void showItem(uint8_t i)
{
	while (vScanKeys());

	uint8_t* resources[NUM_RESOURCES];
	initResources(resources, NUM_RESOURCES);

	LoadPackedRes(MENUBOX, MENUBOX_UNPACKED_SIZE, &resources[MENUBOX]);
	LoadPackedRes(MOONROCK + i, MOONROCK_UNPACKED_SIZE, &resources[MOONROCK + i]);
	
	vClearScreen(0);

	vDrawObject(width/2 - MENUBOX_WIDTH/2, height/2 - MENUBOX_HEIGHT/2, resources[MENUBOX]);
	uint8_t textX = width/2 - MENUBOX_WIDTH/2 + MENUBOX_START_TEXT_X;
	uint8_t textY = height/2 - MENUBOX_HEIGHT/2 + MENUBOX_START_TEXT_Y;

	vPrint(MODE_TRANS, width/2 - (vStrLen(legendaryTreasures[i*2])*THEFONT_WIDTH)/2, textY, legendaryTreasures[i*2]);
	textY += THEFONT_HEIGHT;
	vDrawObject(width/2 - MOONROCK_WIDTH/2, textY, resources[MOONROCK+i]);
	textY += MOONROCK_HEIGHT;
	textY += 3;

	//Draw the text
	uint8_t extreme = width/2 + MENUBOX_WIDTH/2 - MENUBOX_START_TEXT_X;
	printWordWrap(legendaryTreasures[i*2+1], textX, textY, extreme, THEFONT_WIDTH, THEFONT_HEIGHT);

	vFlipScreen(1);

	waitForKeyPress();
	
	freeAllResources(resources, NUM_RESOURCES);
}

void collection()
{
	uint8_t* resources[NUM_RESOURCES];
	initResources(resources, NUM_RESOURCES);

	LoadPackedRes(MENUBOX, MENUBOX_UNPACKED_SIZE, &resources[MENUBOX]);
	LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);

	uint8_t level = 1;
	
	
	
	while (1)
	{
		uint8_t i;
		for (i = 0; i < 4; i++)
			if (keyPress('1' + i) && treasurecollection[ (level-1)*4 + i])
				showItem( (level-1)*4 + i );

		if (keyPress('5'))
			level++;
		
		if (level == 6)
			break;
		
		if (keyPress('*'))
		{
			if (prompt("WOULD YOU LIKE TO CLEAR THE COLLECTION", resources) == 1)
			{
				//Clear the treasurecollection array and write the new values to the file
				for (i = 0; i < NUM_TREASURES; i++)
					treasurecollection[i] = 0;

				int32_t handle = vStreamOpen("collection.txt", STREAM_FILE | STREAM_CREATE | STREAM_WRITE | STREAM_TRUNC | STREAM_BINARY );
				vStreamWrite(handle, treasurecollection, NUM_TREASURES);
				vStreamClose(handle);

				break;
			}
		}

		//Display the changes
		vClearScreen(0); 
		
		vDrawObject(width/2 - MENUBOX_WIDTH/2, height/2 - MENUBOX_HEIGHT/2, resources[MENUBOX]);
		uint8_t textX = width/2 - MENUBOX_WIDTH/2 + MENUBOX_START_TEXT_X;
		uint8_t textY = height/2 - MENUBOX_HEIGHT/2 + MENUBOX_START_TEXT_Y;
		 
		vSetForeColor(vRGB(0,255,0));
		vPrint(MODE_TRANS, width/2 - (THEFONT_WIDTH*10)/2, textY, "COLLECTION");
		textY += THEFONT_HEIGHT;
		vSetForeColor(vRGB(255,0,0));
		vPrint(MODE_TRANS, width/2 - (THEFONT_WIDTH*7)/2, textY, "FLOOR-");
		char buffer[256];
		vitoa(level, buffer, 1, ' ');
		vPrint(MODE_TRANS, width/2 - (THEFONT_WIDTH*7)/2 + THEFONT_WIDTH*6, textY, buffer);
		textY += THEFONT_HEIGHT * 2;

		vSetForeColor(255);
		
		for (i = 0; i < 4; i++)
		{
			vitoa(i+1, buffer, 1, ' ');
			
			vPrint(MODE_TRANS, textX, textY, buffer);
			vPrint(MODE_TRANS, textX + THEFONT_WIDTH, textY, ".");

			if (treasurecollection[ (level-1)*4 + i])
			{
				vPrint(MODE_TRANS, textX + THEFONT_WIDTH*3, textY, legendaryTreasures[(level-1) * 8 + i * 2]);
			}

			textY += THEFONT_HEIGHT;
		}
		
		if (level == 5)
			vPrint(MODE_TRANS, textX, textY, "5. EXIT");
		else
			vPrint(MODE_TRANS, textX, textY, "5. MORE");

		vFlipScreen(1);
	}

	freeAllResources(resources, NUM_RESOURCES);
	
}
