#include "game.h"

#include <vmgp.h>
#include <vstream.h>
#include "rmg.h"
#include "res.h"
#include "globals.h"
#include "app.h"
#include "inventory.h"
#include "victory.h"
#include "gameover.h"

#include "loading.h"

void chooseSeeds()
{
	vSetRandom(vGetTickCount());

	uint8_t i;
	for (i = 0; i < NUM_LEVELS; i++)
		seeds[i] = vGetRandom();
}

uint8_t addItemToInventory(uint8_t item, uint8_t uid)
{
	uint8_t i;
	for (i = 0; i < MAX_INVENTORY; i++)
		if (inventory[i].type == -1)
		{
			inventory[i] = items[item];
			inventory[i].uid = uid;
			
			return 1;
		}

	return 0;
}

void setSpriteLocation(uint8_t sprite, int16_t x, int16_t y)
{
	sprites[sprite].x = x * TILESIZE;
	sprites[sprite].y = y * TILESIZE;
}

void placeSprites(uint8_t* map, uint8_t level, uint8_t prevLevel, uint8_t* resources[])
{
	uint8_t spriteLocation = 1;
	uint8_t i, x, y;

	vSetRandom(seeds[level]);

	for (i = 0; i < MAX_SPRITES; i++)
		sprites[i].isAlive = 0;
	
	//Go ahead and set the player's stats.  We may need them later
	if (prevLevel == 0)
		sprites[0] = unitstats[0];

	//Place the stairs down
	getValidLocation(map, &x, &y);
	sprites[spriteLocation] = unitstats[11];
	setSpriteLocation(spriteLocation, x, y);
	spriteLocation++;

	if (level != 5)
	{
		getValidLocation(map, &x, &y);
		sprites[spriteLocation] = unitstats[12];
		setSpriteLocation(spriteLocation, x, y);
		spriteLocation++;
	}

	//Place the treasures down
	for (i = 0; i < NUM_TREASURES_PER_FLOOR; i++)
	{
		uint8_t r = vGetRandom(), r2 = vGetRandom(), r3 = vGetRandom();

		//Use anim to store the unique identifier of this item.  We'll use this uid to determine
		//if we should place an item or if it has already been picked up
		uint8_t j, needToContinue = 0;
		for (j = 0; j < MAX_INVENTORY; j++)
			if ( level * 4 + i == inventory[j].uid && inventory[j].type != -1)
				needToContinue = 1;

		if (needToContinue) continue;

		getValidLocation(map, &x, &y);
		sprites[spriteLocation + i] = unitstats[10];
		sprites[spriteLocation + i].flag2 = level * 4 + i;
		sprites[spriteLocation + i].isAlive = 1;
		sprites[spriteLocation + i].isVisible = 1;
		setSpriteLocation(spriteLocation + i, x, y);
		
		uint8_t type = r3%5;
		
		if (type == WEAPON || type == ARMOR)
			sprites[spriteLocation + i].flag = validEquipment[level-1][r%validEquipment[(level-1)][0] + 1];
		else
			sprites[spriteLocation + i].flag = r2%(ITEM_END-ACCESSORY_START) + ACCESSORY_START;
	}

	spriteLocation += NUM_TREASURES_PER_FLOOR;

	for (i = 0; i < NUM_LTREASURES_PER_FLOOR; i++)
	{
		uint8_t r = vGetRandom();

		uint8_t j, needToContinue = 0;
		for (j = 0; j < MAX_INVENTORY; j++)
			if ( level*4 + i + NUM_TREASURES_PER_FLOOR == inventory[j].uid && inventory[j].type != -1)
				needToContinue = 1;

		if (needToContinue) continue;

		getValidLocation(map, &x, &y);
		sprites[spriteLocation + i] = unitstats[10];
		setSpriteLocation(spriteLocation + i, x, y);
		sprites[spriteLocation + i].flag = LTREASURE_START + (level-1)*4 + r%4;
		sprites[spriteLocation + i].flag2 = level*4 + i + NUM_TREASURES_PER_FLOOR;
	}

	spriteLocation += NUM_LTREASURES_PER_FLOOR;

	//Place the enemies down
	uint8_t numEnemies = vGetRandom() % (MAX_ENEMIES_PER_FLOOR - (MIN_ENEMIES_PER_FLOOR-1)) + MIN_ENEMIES_PER_FLOOR;
	for (i = 0; i < numEnemies; i++)
	{
		
		getValidLocation(map, &x, &y);
		
		uint8_t j = vGetRandom()%validEnemies[(level-1)][0];
		j = validEnemies[(level-1)][ j + 1 ];
		j = j - PELLET + 1;

		sprites[spriteLocation + i] = unitstats[j];
		setSpriteLocation(spriteLocation + i, x, y);
		
		if (resources[unitstats[j].id] == NULL)
			LoadPackedRes(unitstats[j].id, PELLET_UNPACKED_SIZE, &resources[unitstats[j].id]);

		/*NOTE: SPECTACLES REMOVED
		//If the spectacles are equipped and this is a mimic, reveal it
		if (sprites[0].equippedAcc != -1 && 
			inventory[sprites[0].equippedAcc].id == SPECTACLES &&
			sprites[spriteLocation + i].id == MIMIC)
				sprites[spriteLocation + i].flag3 = 1;
		*/
	}
	
	spriteLocation += numEnemies;

	//Place the player down (on top of the stairs that lead up)
	if (prevLevel == 0) 
	{
		sprites[0].equippedWep = 0;

		inventory[0] = items[WEAPON_START];
		inventory[1] = items[ITEM_START];
		inventory[2] = items[ITEM_START];
	}

	sprites[0].isAlive = 1;

	if (level > prevLevel)
	{
		sprites[0].x = sprites[1].x;
		sprites[0].y = sprites[1].y;
	}
	else
	{
		sprites[0].x = sprites[2].x;
		sprites[0].y = sprites[2].y;
	}

	//Place the traps
	//Also, if the spectacles are equipped, reveal the trap
	uint8_t numTraps = vGetRandom()%(MAX_TRAPS_PER_FLOOR-(MIN_TRAPS_PER_FLOOR-1))+MIN_TRAPS_PER_FLOOR;

	for (i = 0; i < numTraps; i++)
	{
		getValidLocation(map, &x, &y);
		map[y * MAP_SIZE_X+x] = (vGetRandom()%((TRAP_HEALTH/10)-((TRAP_POISON/10)-1))+(TRAP_POISON)/10) * 10;

		/** NOTE: SPECTACLES REMOVED
		if (sprites[0].equippedAcc != -1 && inventory[sprites[0].equippedAcc].id == SPECTACLES)
		{
			revealTrap(x, y, resources, 0);
		}
		*/
	}

}

void animateSprite(int i)
{
	
	if (sprites[i].type == ENEMY || sprites[i].type == PLAYER)
	{
		if (vGetTickCount() - sprites[i].animationCounter > ANIMATION_RATE)
		{
			if (sprites[i].id == MIMIC && (sprites[i].flag != 0 || sprites[i].flag3 != 0))
			{
				sprites[i].animationCounter = vGetTickCount();

				sprites[i].frame++;
				if (sprites[i].frame == NUM_FRAMES)
					sprites[i].frame = 1;
				
			}
			else if (sprites[i].id != MIMIC)
			{
				sprites[i].animationCounter = vGetTickCount();
				
				if (i == 0 && sprites[i].flag2 == 4)
					return;

				if (!sprites[i].flag3) 
					sprites[i].frame++;
				else
					sprites[i].frame--;

				if (sprites[i].frame == NUM_FRAMES)
				{
					sprites[i].flag3 = 1;
					sprites[i].frame = NUM_FRAMES-1;
				}
				if (sprites[i].frame == -1)
				{
					sprites[i].flag3 = 0;
					sprites[i].frame = 0;
				}
			}
		}
	}
}

void drawSprites(uint8_t* resources[], uint16_t worldX, uint16_t worldY)
{ 
	uint8_t i;
	
	//Draw the rest of the sprites
	for (i = 1; i < MAX_SPRITES; i++)
		if (sprites[i].isAlive && sprites[i].isVisible)
		{
			animateSprite(i);

			blit(	sprites[i].x - worldX, sprites[i].y - worldY, 
					0, sprites[i].frame * SPRITE_HEIGHT,
					SPRITE_WIDTH, SPRITE_HEIGHT, resources[sprites[i].id], width, height);
		}
	
	//Draw the player
	if (sprites[0].isAlive && sprites[0].isVisible)
	{
		animateSprite(0);
		blit(	sprites[0].x - worldX, sprites[0].y - worldY, 
				sprites[0].frame * SPRITE_WIDTH, sprites[0].flag2 * SPRITE_HEIGHT, 
				SPRITE_WIDTH, SPRITE_HEIGHT, 
				resources[sprites[0].id], width, height);
	}
	
}

void render(uint8_t* resources[])
{
	vClearScreen(0);
	vMapSetXY(mapX, mapY);
	vUpdateSpriteMap();
	drawSprites(resources, mapX, mapY);
	
	vDrawObject(width-STATBOX_WIDTH, 0, resources[STATBOX]);
	vSetForeColor(vRGB(255,255,255));
	vPrint(MODE_TRANS, width - THEFONT_WIDTH*7+2, 4, "HP ");

	char buffer[255];
	vitoa(sprites[0].hp, buffer, 5, ' ');
	vPrint(MODE_TRANS, width - THEFONT_WIDTH*7+2 + THEFONT_WIDTH, 4, buffer);

	if (sprites[0].statusTimePoison > 0)	vPrint(MODE_TRANS, width - THEFONT_WIDTH*7+2,   THEFONT_HEIGHT+4, "POISON");
	if (sprites[0].statusTimeConfusion > 0) vPrint(MODE_TRANS, width - THEFONT_WIDTH*7+2, THEFONT_HEIGHT*2+4, "DIZZY");
	if (sprites[0].statusTimePowerup > 0)	vPrint(MODE_TRANS, width - THEFONT_WIDTH*7+2, THEFONT_HEIGHT*3+4, "POWER");

	vFlipScreen(1);
}

void centerMapOnPlayer()
{
	mapX = sprites[0].x - width/2 + SPRITE_WIDTH/2;
	mapY = sprites[0].y - (height)/2 + SPRITE_HEIGHT/2;

	if (mapX < 0) mapX = 0;
	if (mapY < 0) mapY = 0;
	if (mapX > MAP_SIZE_X * TILESIZE - width)  mapX = MAP_SIZE_X * TILESIZE - width;
	if (mapY > MAP_SIZE_Y * TILESIZE - (height)) mapY = MAP_SIZE_Y * TILESIZE - (height);
}

void moveSprite(uint8_t sprite, int8_t vx, int8_t vy, uint8_t* resources[])
{
	uint8_t i;
	for (i = SCROLL_SPEED; i <= TILESIZE; i += SCROLL_SPEED)
	{
		sprites[sprite].x += SCROLL_SPEED * vx;
		sprites[sprite].y += SCROLL_SPEED * vy;

		centerMapOnPlayer();
		render(resources);
	}
}

void killEnemy(uint8_t enemy, uint8_t* resources[])
{
	uint8_t count = 0;
	int32_t start = vGetTickCount();
	while(1)
	{
		if (vGetTickCount() - start > DEATH_FLASH_LENGTH)
		{
			start = vGetTickCount();
			sprites[enemy].isVisible = !sprites[enemy].isVisible;
			count++;
		}

		if (count >= DEATH_FLASH_TIMES) break;

		render(resources);
	}
}

void killPlayer(uint8_t* resources[])
{
	uint8_t count = 0;
	int32_t start = vGetTickCount();
	sprites[0].frame = 0;
	sprites[0].flag2 = 4;
	while(1)
	{
		if (vGetTickCount() - start > DEATH_FLASH_LENGTH*4)
		{
			start = vGetTickCount();
			if (sprites[0].frame == 2) sprites[0].frame = 0;
			else sprites[0].frame = 2;

			count++;
		}
		render(resources);

		if (count >= DEATH_FLASH_TIMES+1) break;
	}
	displayMessage("YOU ARE DEAD.  GAME OVER.", resources);
	playerIsDead = 1;
}

void hurtEnemy(uint8_t enemy, uint8_t* resources[])
{
	//Perform the necessary graphical fluff associated with an attack
	if (doesVibrate) vSetVibrate(1, FLASH_LENGTH);
	flash(vRGB(255,0,0), FLASH_LENGTH);
	if (sound) vBeep(659, FLASH_LENGTH);
	render(resources);

	if (vGetRandom()%100 < PERCENT_CHANGE_TO_MISS)
	{
		displayMessage("DARYL MISSES", resources);
		return;
	}

	//Do the actual calculations
	int16_t damage = 1;
	
	if (sprites[0].equippedWep != -1 && inventory[sprites[0].equippedWep].type != -1 )
	{
		damage = (inventory[sprites[0].equippedWep].strength - sprites[enemy].defense);
		damage += vGetRandom()%3 - 1;
		if (damage < 1) damage = 1;

		//Handle the chaos blade
		if (inventory[sprites[0].equippedWep].id == CHBLADE)
		{
			sprites[0].hp -= 4;
			displayMessage("THE CHAOS BLADE DRAINS 4 HP FROM YOU", resources);
		}
	}

	if (sprites[0].statusTimePowerup > 0) damage *= 2;

	sprites[enemy].hp -= damage;
	
	char buffer[255];
	vitoa(damage, buffer, 2, ' ');
	char message[255] = "DARYL ATTACKED ";
	strcat(message, unitstats[sprites[enemy].id - PELLET + 1].name);
	strcat(message, " FOR ");
	strcat(message, buffer);
	strcat(message, " DAMAGE");
	displayMessage(message, resources);

	if (sprites[enemy].hp <= 0) 
	{
		killEnemy(enemy, resources);
		sprites[enemy].isAlive = 0;
		sprites[enemy].isVisible = 0;
		sprites[enemy].flag3 = TURNS_TO_RESPAWN;
	}

}

void handleRespawn(uint8_t* resources[])
{
	uint8_t i;
	for (i = 1; i < MAX_SPRITES; i++)
		if (sprites[i].type == ENEMY && !sprites[i].isAlive && sprites[i].flag3 > 0)
		{
			sprites[i].flag3--;

			if (sprites[i].flag3 == 1)
			{
				uint8_t x, y;

				uint8_t j = vGetRandom()%validEnemies[(level-1)][0];
				j = validEnemies[(level-1)][ j + 1 ];
				j = j - PELLET + 1;

				getValidLocation(map, &x, &y);
				sprites[i] = unitstats[j];
				if (resources[sprites[i].id] == NULL)
					LoadPackedRes(sprites[i].id, PELLET_UNPACKED_SIZE, &resources[sprites[i].id]);

				setSpriteLocation(i, x, y);
				sprites[i].isAlive = 1; 
				sprites[i].isVisible = 1;
			}

		}
}

uint8_t player(uint8_t* resources[], uint8_t* map)
{
	if (playerIsDead) return;

	uint8_t walking = 0;
	vanish = 0;
	const uint8_t floor = 2;

	uint8_t x, y;
	
	uint8_t hasMoved = 0, hasBeenDone;
	uint8_t tx, ty;
	int8_t ret = -1;
	while (ret == -1)
	{
		uint8_t collision[3][3];
		
		for (x = 0; x < 3; x++) for (y = 0; y < 3; y++) collision[x][y] = 0;

		for (x = 1; x < MAX_SPRITES; x++)
			if (sprites[x].isAlive)
			{
				if (1+(sprites[x].x-sprites[0].x)/TILESIZE >= 0 &&
					1+(sprites[x].x-sprites[0].x)/TILESIZE < 3 &&
					1+(sprites[x].y-sprites[0].y)/TILESIZE >= 0 &&
					1+(sprites[x].y-sprites[0].y)/TILESIZE < 3)
				{
					collision[ 1+(sprites[x].x-sprites[0].x)/TILESIZE ][ 1+(sprites[x].y-sprites[0].y)/TILESIZE ] = sprites[x].type * 50 + x;
				}
			}

		hasBeenDone = 1;
		uint32_t keys = vGetButtonData();
		
		/** DEBUG MODE REMOVED
		if (debug)
		{
			uint8_t k;
			for (k = 0; k < 26; k++)
				if (keyPress('a'+k)) 
					if (!(keys&KEY_FIRE)) addItemToInventory(k, 100);
					else addItemToInventory(k+26, 100);
				
		}
		*/

		//Player's tile position
		tx = (sprites[0].x+SPRITE_WIDTH/2) / TILESIZE;
		ty = (sprites[0].y+SPRITE_HEIGHT/2) / TILESIZE;

		uint8_t neighbor, sprite;

		int8_t cx = 0, cy = 0;
		if (keys & KEY_LEFT)
			cx = -1;
		else if (keys & KEY_RIGHT)
			cx = 1;
		else if (keys & KEY_UP)
			cy = -1;
		else if (keys & KEY_DOWN)
			cy = 1;
		
		//Apply confusion effect
		if (cx != 0 || cy != 0)
		{
			if (sprites[0].statusTimeConfusion > 0 && (sprites[0].equippedAcc == -1 || (inventory[sprites[0].equippedAcc].id != HCROSS && inventory[sprites[0].equippedAcc].id != IRONMASK)))
			{
				cx = cy = 0;
				if (vGetRandom()%2)
				{
					if (vGetRandom()%2) cx = -1;
					else cx = 1;
				}
				else
				{
					if (vGetRandom()%2) cy = -1;
					else cy = 1;
				}
					
			}
		}

		if ( cx == -1) sprites[0].flag2 = 2;
		if ( cx == 1 ) sprites[0].flag2 = 3;
		if ( cy == -1) sprites[0].flag2 = 1;
		if ( cy == 1 ) sprites[0].flag2 = 0;

		if (cx != 0 || cy != 0)
		{
			neighbor = collision[ cx+1 ][ cy+1 ] / 50;
			uint8_t sprite = collision[ cx + 1 ][ cy + 1 ] % 50;


			if (neighbor == ENEMY && sprites[sprite].isAlive)
			{
				hurtEnemy(sprite, resources);

				if (sprites[sprite].id == MIMIC) sprites[sprite].flag = sprites[sprite].x;

				ret = 1;
			}
			else if (neighbor == TREASURE)
			{
				if (addItemToInventory( sprites[sprite].flag, sprites[sprite].flag2))
				{
					uint8_t pf = sprites[0].frame, pa = sprites[0].flag2;

					sprites[0].frame = 1;
					sprites[0].flag2 = 4;
					render(resources);
					if (sound) vBeep(300, FLASH_LENGTH);
					displayMessage(items[sprites[sprite].flag].name, resources);

					sprites[0].frame = pf;
					sprites[0].flag2 = pa;

					sprites[sprite].isAlive = 0;
				}
				else
					displayMessage("YOUR INVENTORY IS FULL", resources);

				ret = 1;
			}
			else if (map[ (ty+cy) * MAP_SIZE_X + (tx+cx) ] == floor || map[ (ty+cy) * MAP_SIZE_X + (tx+cx) ] >= TRAP_POISON)
			{
				moveSprite(0, cx, cy, resources);

				if (sprites[0].equippedAcc != -1 && inventory[sprites[0].equippedAcc].id == HBOOT)
					hasMoved++;
				else hasMoved = 2;

				hasBeenDone = 0;
			}
			
			handleRespawn(resources);

			if (hasMoved == 2)
				ret = 1;
		}
		
		
		if (keyPress('*'))
		{
			if (prompt("ARE YOU SURE YOU WANT TO QUIT", resources) == 1)
			{
				saveGame();
				ret = 0;
			}
		}
		if (keyPress('5')) 
		{
			freeAllResources(resources, NUM_RESOURCES);
			inv();

			LoadPackedRes(TILES, TILES_UNPACKED_SIZE, &resources[TILES]);
			LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);
			LoadPackedRes(DARYL, DARYL_UNPACKED_SIZE, &resources[DARYL]);
			LoadPackedRes(CHEST, CHEST_UNPACKED_SIZE, &resources[CHEST]);
			LoadPackedRes(STAIRSUP, STAIRSUP_UNPACKED_SIZE, &resources[STAIRSUP]);
			LoadPackedRes(STAIRSDOWN, STAIRSDOWN_UNPACKED_SIZE, &resources[STAIRSDOWN]);
			LoadPackedRes(STATBOX, STATBOX_UNPACKED_SIZE, &resources[STATBOX]);

			uint8_t i;
			for (i = 1; i < MAX_SPRITES; i++)
			{
				if (sprites[i].isAlive || sprites[i].flag3 != 0)
				{
					if (resources[sprites[i].id] == NULL)
						LoadPackedRes(sprites[i].id, PELLET_UNPACKED_SIZE, &resources[sprites[i].id]);
				}

				if (sprites[i].isAlive && sprites[i].type == ENEMY && sprites[i].hp <= 0)
				{
					sprites[i].isAlive = 0;
					sprites[i].flag3 = TURNS_TO_RESPAWN;
				}
			}

			if (sprites[0].hp <= 0)
			{
				killPlayer(resources);
				break;
			}

			if (vanish)
			{
				level = 1;
				ret = 2;
			}
		}

		if (hasMoved && !hasBeenDone)
		{
			
			tx = (sprites[0].x) / TILESIZE;
			ty = (sprites[0].y) / TILESIZE;
			
			//Player moves on stairs
			uint8_t pos = ty * MAP_SIZE_X + tx;
			
			if (collision[1+cx][1+cy]/50 == STAIRS_UP)
				if (prompt("DO YOU WANT TO USE THE STAIRS", resources) == 1)
					ret = 2;
			if (collision[1+cx][1+cy]/50 == STAIRS_DOWN)
				if (prompt("DO YOU WANT TO USE THE STAIRS", resources) == 1)
					ret = 3;

			revealTrap(tx, ty, resources, 1);
			if (sprites[0].hp <= 0)
			{
				killPlayer(resources);
				break;
			}

			//Handle things that happen every time the player moves
			if (sprites[0].statusTimePoison > 0)
			{
				sprites[0].statusTimePoison--;
				if ( hasMoved == 2 && sprites[0].equippedAcc == -1 || (inventory[sprites[0].equippedAcc].id != HCROSS && inventory[sprites[0].equippedAcc].id != MANDRAKE))
					sprites[0].hp--;
			}
			if (sprites[0].statusTimeConfusion > 0)
			{
				sprites[0].statusTimeConfusion--;
			}
			if (sprites[0].statusTimePowerup > 0 && hasMoved == 2)
			{
				sprites[0].statusTimePowerup--;
			}
			//Handle breaking of Hermes boots
			if (sprites[0].equippedAcc != -1 && inventory[sprites[0].equippedAcc].id == HBOOT)
			{
				inventory[sprites[0].equippedAcc].hp--;

				if (inventory[sprites[0].equippedAcc].hp <= 0)
				{
					inventory[sprites[0].equippedAcc].type = -1;
					sprites[0].equippedAcc = -1;
					displayMessage("YOU WORE OUT THE HERMES BOOTS", resources);
				}
			}
			if (hasMoved == 2 && sprites[0].equippedArm != -1 && inventory[sprites[0].equippedArm].id == DARMOR)
			{
				sprites[0].hp--;
			}
			if (sprites[0].equippedArm != -1 && inventory[sprites[0].equippedArm].id == HARMOR)
			{
				sprites[0].hp++;
				if (sprites[0].hp > 100) sprites[0].hp = 100;
			}
			
		}

		if (sprites[0].hp <= 0 && sprites[0].flag2 != 4)
		{
			sprites[0].flag2 = 4;
			sprites[0].frame = 0;
		}
		
		centerMapOnPlayer();
		render(resources);

		if (demo && vGetTickCount() - demoTime > 1000 * 60 * DEMO_TIME)
		{
			ret = 1;
			break;
		}
	}
	
	return ret;
}

void hurtPlayer(uint8_t enemy, uint8_t * resources[])
{
	if (doesVibrate) vSetVibrate(1, FLASH_LENGTH);
	flash(vRGB(255,0,0), FLASH_LENGTH);
	if (sound) vBeep(659, FLASH_LENGTH);
	render(resources);

	uint8_t pa = sprites[0].flag2, pf = sprites[0].frame;

	if (vGetRandom()%100 < PERCENT_CHANGE_TO_MISS)
	{
		char message[255] = "";
		strcat(message, unitstats[sprites[enemy].id - PELLET + 1].name);
		strcat(message, " MISSES");
		displayMessage(message, resources);
		return;
	}

	sprites[0].flag2 = 4;
	sprites[0].frame = 0;
	render(resources);

	uint8_t defense = sprites[enemy].strength;
	if (sprites[0].equippedArm != -1 && inventory[sprites[0].equippedArm].type != -1)
		defense = inventory[sprites[0].equippedArm].strength;

	if (sprites[0].statusTimePowerup > 0) defense *= 2;
	
	uint8_t dragonDef = 0;
	//Apply defense of Dragon's Skull
	if (sprites[0].equippedAcc != -1 && inventory[sprites[0].equippedAcc].id == DSKULL)
		if (sprites[enemy].id == DRAGON) dragonDef = sprites[enemy].strength/2;

	int8_t damage = sprites[enemy].strength - sprites[enemy].strength/defense - dragonDef;
	damage += vGetRandom()%3 - 1;

	if (damage < 1) damage = 1;
	sprites[0].hp -= damage;

	char buffer[255];
	vitoa(damage, buffer, 2, ' ');
	char message[255] = "";
	strcat(message, unitstats[sprites[enemy].id - PELLET + 1].name);
	strcat(message, " ATTACKED DARYL FOR ");
	strcat(message, buffer);
	strcat(message, " DAMAGE");
	displayMessage(message, resources);

	sprites[0].frame = pf;
	sprites[0].flag2 = pa;

	if (sprites[0].hp <= 0) killPlayer(resources);
	
}

void enemies(uint8_t* resources[])
{
	
	uint8_t* collisionMap = vNewPtr(MAP_SIZE_X * MAP_SIZE_Y * sizeof(uint8_t));
	memset( (void*)collisionMap, 0, MAP_SIZE_X * MAP_SIZE_Y * sizeof(uint8_t));
	uint8_t x, y, i, j;
	
	for (x = 0; x < MAX_SPRITES; x++)
		if (sprites[x].isAlive && sprites[x].type != STAIRS_UP && sprites[x].type != STAIRS_DOWN)
			collisionMap[ ((sprites[x].y)/TILESIZE) * MAP_SIZE_X + ((sprites[x].x)/TILESIZE) ] = (x+1);

	
	//Flag enemies to be moved
	for (i = 1; i < MAX_SPRITES; i++)
	{
		if (sprites[i].isAlive && sprites[i].isVisible && sprites[i].type == ENEMY &&
			sprites[i].id != MIMIC &&
			(sprites[i].x - mapX > -SPRITE_WIDTH && sprites[i].x - mapX < width &&
			 sprites[i].y - mapY > -SPRITE_HEIGHT && sprites[i].y - mapY < height) || 
			sprites[i].flag != 0)
		{
			if (sprites[i].type == ENEMY)
			{
				sprites[i].flag = sprites[i].x;
				sprites[i].flag2 = sprites[i].y;
			}
		}
		
		//Attack if appropriate, and if we attack cut out of the function
		if (sprites[i].type == ENEMY && sprites[i].isAlive)
		{
			uint8_t px = sprites[0].x / TILESIZE,
					py = sprites[0].y / TILESIZE,
					ex = sprites[i].flag / TILESIZE,
					ey = sprites[i].flag2 / TILESIZE;

			if (collisionMap[ ey * MAP_SIZE_X + (ex + 1) ] == 1 ||
				collisionMap[ ey * MAP_SIZE_X + (ex - 1) ] == 1 ||
				collisionMap[ (ey+1) * MAP_SIZE_X + ex ] == 1 ||
				collisionMap[ (ey-1) * MAP_SIZE_X + ex ] == 1)
			{
				hurtPlayer(i, resources);
				if (playerIsDead) break;
			}
		}
	}
	
	for (i = SCROLL_SPEED; i <= TILESIZE; i += SCROLL_SPEED)
	{
		if (playerIsDead) break;

		uint8_t mr = 0;
		for (j = 1; j < MAX_SPRITES; j++)
		{
			if (sprites[j].flag == 0 || sprites[j].type != ENEMY || sprites[j].isAlive == 0) continue;
			
			mr = 1;

			uint8_t px = sprites[0].x / TILESIZE,
					py = sprites[0].y / TILESIZE,
					ex = sprites[j].flag / TILESIZE,
					ey = sprites[j].flag2 / TILESIZE;

			if (ex < px && 
				(collisionMap[ (ey) * MAP_SIZE_X + (ex+1) ] == 0 || collisionMap[ (ey) * MAP_SIZE_X + (ex+1) ] == j+1) &&
				map[ (ey) * MAP_SIZE_X + (ex+1) ] == FLOOR )
			{
				collisionMap[ (ey) * MAP_SIZE_X + (ex+1) ] = j + 1;
				sprites[j].x += SCROLL_SPEED;
			}
			else if (ex > px &&
					 (collisionMap[ (ey) * MAP_SIZE_X + (ex-1) ] == 0 || collisionMap[ (ey) * MAP_SIZE_X + (ex-1) ] == j+1) &&
					 map[ (ey) * MAP_SIZE_X + (ex-1) ] == FLOOR)
			{
				collisionMap[ (ey) * MAP_SIZE_X + (ex-1) ] = j + 1;
				sprites[j].x -= SCROLL_SPEED;
			}
			else if (ey < py &&
					 (collisionMap[ (ey+1) * MAP_SIZE_X + (ex) ] == 0 || collisionMap[ (ey+1) * MAP_SIZE_X + (ex) ] == j+1) &&
					 map[ (ey+1) * MAP_SIZE_X + (ex) ] == FLOOR)
			{
				collisionMap[ (ey+1) * MAP_SIZE_X + (ex) ] = j + 1;
				sprites[j].y += SCROLL_SPEED;
			}
			else if (ey > py &&
					 (collisionMap[ (ey-1) * MAP_SIZE_X + (ex) ] == 0 || collisionMap[ (ey-1) * MAP_SIZE_X + (ex) ] == j+1) &&
					 map[ (ey-1) * MAP_SIZE_X + (ex) ] == FLOOR)
			{
				collisionMap[ (ey-1) * MAP_SIZE_X + (ex) ] = j + 1;
				sprites[j].y -= SCROLL_SPEED;
			}
			
		}

		if (mr) render(resources);
	}
	
	vDisposePtr(collisionMap);
	
}

void game(uint8_t cont)
{
	playerIsDead = 0;

	loading();
	
	uint8_t* resources[NUM_RESOURCES];
	initResources(resources, NUM_RESOURCES);

	LoadPackedRes(TILES, TILES_UNPACKED_SIZE, &resources[TILES]);
	LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);
	LoadPackedRes(DARYL, DARYL_UNPACKED_SIZE, &resources[DARYL]);
	LoadPackedRes(CHEST, CHEST_UNPACKED_SIZE, &resources[CHEST]);
	LoadPackedRes(STAIRSUP, STAIRSUP_UNPACKED_SIZE, &resources[STAIRSUP]);
	LoadPackedRes(STAIRSDOWN, STAIRSDOWN_UNPACKED_SIZE, &resources[STAIRSDOWN]);
	LoadPackedRes(STATBOX, STATBOX_UNPACKED_SIZE, &resources[STATBOX]);

	MAP_HEADER header;
	
	if (!cont)
	{
		level = 1;

		chooseSeeds();

		map = generateRandomMap(&header, MAP_SIZE_X, MAP_SIZE_Y, seeds[(level-1)], &floor1, &floor2);
		header.format = VCAPS_RGB332;
		header.tiledata = resources[TILES];
		
		vMapInit(&header);
		
		uint8_t i;
		for (i = 0; i < MAX_INVENTORY; i++) inventory[i].type = -1;

		
		placeSprites(map, level, 0, resources);
		
		
	}
	else
	{
		loadGame(resources);
		
		map = generateRandomMap(&header, MAP_SIZE_X, MAP_SIZE_Y, seeds[level-1], &floor1, &floor2);
		header.format = VCAPS_RGB332;
		header.tiledata = resources[TILES];
		vMapInit(&header);
	}

	if (sound)
	{
		vDisposePtr(music);
		LoadPackedRes(MDUNGEON1, MDUNGEON1_UNPACKED_SIZE, &music);
		vPlayResource(music, MDUNGEON1_UNPACKED_SIZE, SOUND_TYPE_MIDI);
	}

	while(1)
	{ 
		uint8_t ret = player(resources, map);
		
		if (ret == 0) break;	//Player has quit.  Data has already been saved
		if (ret == 1);			//Do nothing

		if (ret == 2 || ret == 3 )
		{
			loading();
			freeAllResources(resources, NUM_RESOURCES);
			LoadPackedRes(TILES, TILES_UNPACKED_SIZE, &resources[TILES]);
			LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);
			LoadPackedRes(DARYL, DARYL_UNPACKED_SIZE, &resources[DARYL]);
			LoadPackedRes(CHEST, CHEST_UNPACKED_SIZE, &resources[CHEST]);
			LoadPackedRes(STAIRSUP, STAIRSUP_UNPACKED_SIZE, &resources[STAIRSUP]);
			LoadPackedRes(STAIRSDOWN, STAIRSDOWN_UNPACKED_SIZE, &resources[STAIRSDOWN]);
			LoadPackedRes(STATBOX, STATBOX_UNPACKED_SIZE, &resources[STATBOX]);
		}

		if (ret == 2)			//Go a level up
		{
			level--;
			if (level == 0)		//If we go back to the top, save the treasures and return to the menu
			{
				//Add appropriate items to the collection
				uint8_t i, nTreasures = 0;
				for (i = 0; i < MAX_INVENTORY; i++)
					if (inventory[i].type == LTREASURE)
					{
						treasurecollection[inventory[i].id - MOONROCK] = 1;
						nTreasures++;
					}

				//Save the collection
				saveCollection();
				
				//Delete the save
				int32_t handle = vStreamOpen("save.txt", STREAM_FILE | STREAM_WRITE | STREAM_CREATE | STREAM_TRUNC | STREAM_BINARY );
				vStreamClose(handle);

				if (nTreasures > 0)
					victory(resources, nTreasures);

				break;
			}
			else
			{
				vDisposePtr(map);
				vDisposePtr(header.mapoffset);
				vMapDispose();
				map = generateRandomMap(&header, MAP_SIZE_X, MAP_SIZE_Y, seeds[level - 1], &floor1, &floor2);
				header.format = VCAPS_RGB332;
				header.tiledata = resources[TILES];
				vMapInit(&header);

				placeSprites(map, level, level+1, resources);

				if (sound)
				{
					vDisposePtr(music);
					LoadPackedRes( musicres[ (level-1) * 2 ], musicres[ (level-1) * 2 + 1 ], &music );
					vPlayResource( music, musicres[ (level-1) * 2 + 1 ], SOUND_TYPE_MIDI);
				}
			}
		}
		if (ret == 3)			//Go a level down
		{
			level++;
			vDisposePtr(map);
			vDisposePtr(header.mapoffset);
			vMapDispose();
			map = generateRandomMap(&header, MAP_SIZE_X, MAP_SIZE_Y, seeds[level - 1], &floor1, &floor2);
			header.format = VCAPS_RGB332;
			header.tiledata = resources[TILES];
			vMapInit(&header);
			
			placeSprites(map, level, level-1, resources);

			if (sound)
			{
				vDisposePtr(music);
				LoadPackedRes( musicres[ (level-1) * 2 ], musicres[ (level-1) * 2 + 1 ], &music );
				vPlayResource( music, musicres[ (level-1) * 2 + 1 ], SOUND_TYPE_MIDI );
			}
		}
			
		if (!playerIsDead && ret == 1)
			enemies(resources);
		
		if (playerIsDead)
		{
			gameover(resources);
			break;
		}

		if (demo && vGetTickCount() - demoTime > 1000 * 60 * DEMO_TIME) break;
	}
	
	vDisposePtr(map);
	vDisposePtr(header.mapoffset);
	
	if (sound) 
	{
		vDisposePtr(music);
		music = NULL;
	}

	vMapDispose();
	freeAllResources(resources, NUM_RESOURCES);
}
