#include "inventory.h"

#include <vmgp.h>
#include "globals.h"
#include "res.h"
#include "app.h"
#include "utility.h"
#include "inventoryitem.h"


void useConsumable(char *name, uint8_t* resources[])
{
	if (!strcmp(name, "GREEN POTION"))
	{
		sprites[0].hp += 50;
		flash(vRGB(0,255,0), FLASH_LENGTH);
		displayMessage("YOU RECOVERED 50 HP", resources);
	}
	else if (!strcmp(name, "BLUE POTION"))
	{
		flash(vRGB(0,0,255), FLASH_LENGTH);
		displayMessage("YOU FEEL COMPLETELY HEALED FROM ALL AILMENTS", resources);
		sprites[0].statusTimeConfusion = 0;
		sprites[0].statusTimePoison = 0;
		sprites[0].statusTimePowerup = 0;
	}
	else if (!strcmp(name, "PURPLE POTION"))
	{
		sprites[0].hp -= 25;
		flash(vRGB(255,0,255), FLASH_LENGTH);
		displayMessage("YOU FEEL SICK AND LOSE 25 HP", resources);
	}
	else if (!strcmp(name, "RED POTION"))
	{
		sprites[0].hp = 100;
		flash(vRGB(255,0,0), FLASH_LENGTH);
		displayMessage("YOU COMPLETELY RECOVERED YOUR HP", resources);
	}
	else if (!strcmp(name, "YELLOW POTION"))
	{
		sprites[0].statusTimeConfusion = CONFUSION_LENGTH;
		flash(vRGB(255,255,0), FLASH_LENGTH);
		displayMessage("YOU START TO FEEL DIZZY", resources);
	}
	else if (!strcmp(name, "ORANGE POTION"))
	{
		uint8_t i;
		for ( i = 1; i < MAX_SPRITES; i++)
			if (sprites[i].isAlive && sprites[i].type == ENEMY)
			{
				if (sprites[i].x - mapX > -SPRITE_WIDTH && sprites[i].x - mapX < width &&
					sprites[i].y - mapY > -SPRITE_HEIGHT && sprites[i].y - mapY < height)
						sprites[i].isAlive = 0;
			}

		flash(vRGB(255,128,0), FLASH_LENGTH);
		displayMessage("ALL THE ENEMIES SURROUNDING YOU ARE GONE", resources);
	}
	else if (!strcmp(name, "WHITE POTION"))
	{
		sprites[0].statusTimePoison = POISON_LENGTH;
		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("YOU FEEL VERY SICK... THE POTION WAS POISON", resources);
	}
	else if (!strcmp(name, "BLACK POTION"))
	{
		useConsumable( items[ vGetRandom()%( (ITEM_END-1) - (ITEM_START-1) ) + ITEM_START ].name, resources);
	}
	else if (!strcmp(name, "WEAPON POWDER"))
	{
		if (sprites[0].equippedWep != -1 && inventory[sprites[0].equippedWep].type != -1)
			inventory[sprites[0].equippedWep].hp = items[ inventory[sprites[0].equippedWep].id - DIRK ].hp;

		displayMessage("YOUR WEAPON IS FULLY RESTORED", resources);

	}
	else if (!strcmp(name, "SCROLL OF SORROW"))
	{
		uint8_t i;
		for ( i = 0; i < MAX_SPRITES; i++)
			if (sprites[i].isAlive && (sprites[i].type == ENEMY || sprites[i].type == PLAYER))
			{
				if (sprites[i].x - mapX > -SPRITE_WIDTH && sprites[i].x - mapX < width &&
					sprites[i].y - mapY > -SPRITE_HEIGHT && sprites[i].y - mapY < height)
						sprites[i].hp -= 10;
			}

		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("A SHARP PAIN IS FELT BY YOU AND ALL AROUND YOU", resources);
	}
	else if (!strcmp(name, "SCROLL OF EXPLOSION"))
	{
		uint8_t i;
		for ( i = 1; i < MAX_SPRITES; i++)
			if (sprites[i].isAlive && sprites[i].type == ENEMY)
			{
				if (sprites[i].x - mapX > -SPRITE_WIDTH && sprites[i].x - mapX < width &&
					sprites[i].y - mapY > -SPRITE_HEIGHT && sprites[i].y - mapY < height)
						sprites[i].hp -= 15;
			}

		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("AN EXPLOSION ROCKS THE ROOM DAMAGING THOSE AROUND YOU", resources);
	}
	else if (!strcmp(name, "SCROLL OF TELEPORT"))
	{
		uint8_t x, y;
		getValidLocation(map, &x, &y);
		sprites[0].x = x * TILESIZE;
		sprites[0].y = y * TILESIZE;

		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("YOU DISAPPEAR AND REAPPEAR IN A NEW LOCATION", resources);
	}
	else if (!strcmp(name, "SCROLL OF WISDOM"))
	{
		//Reveal every trap & mimic
		uint8_t x, y;
		for (x = 0; x < MAP_SIZE_X; x++)
			for (y = 0; y < MAP_SIZE_Y; y++)
				revealTrap(x, y, resources, 1);

		for (x = 0; x < MAX_SPRITES; x++)
			if (sprites[x].isAlive && sprites[x].isVisible && sprites[x].id == MIMIC)
				sprites[x].flag = 1;

		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("YOU SUDDENLY SEE THINGS AS THEY TRULY ARE", resources);
	}
	else if (!strcmp(name, "SCROLL OF POWER"))
	{
		sprites[0].statusTimePowerup = 10;
		
		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("YOU SUDDENLY FEEL MUCH STRONGER", resources);
	}
	else if (!strcmp(name, "SCROLL OF MYSTERY"))
	{
		useConsumable( items[ vGetRandom()%(SCROLL_END-(SCROLL_START-1)) + SCROLL_START ].name, resources);
	}
	else if (!strcmp(name, "WARP STONE"))
	{
		flash(vRGB(255,255,255), FLASH_LENGTH);
		displayMessage("YOU DISAPPEAR AND REAPPEAR OUTSIDE THE DUNGEON", resources);
		vanish = 1;
	}

	if (sprites[0].hp > 100) sprites[0].hp = 100;
	
}

void changeEquipment(uint8_t pos, int8_t e, uint8_t* resources[])
{
	int8_t prev = sprites[0].equippedAcc;

	int16_t prevStrength = 0, strength = 0, shouldDisp = 1;
	char buffer[50] = "", numbuffer[50];

	if (inventory[pos].type == WEAPON)
	{
		strcat(buffer, "STRENGTH ");

		if (sprites[0].equippedWep != -1) 
		{
			prevStrength = inventory[sprites[0].equippedWep].strength;
		}

		if (e == -1 && pos != sprites[0].equippedWep)
			shouldDisp = 0;
		else
			sprites[0].equippedWep = e;

		if (e != -1)  strength = inventory[sprites[0].equippedWep].strength;
	}
	if (inventory[pos].type == ARMOR)
	{
		strcat(buffer, "DEFENSE ");

		if (sprites[0].equippedArm != -1) 
		{
			prevStrength = inventory[sprites[0].equippedArm].strength;

			if (prevStrength == 2) prevStrength = 8;
			else if (prevStrength == 8) prevStrength = 2;
		}
		
		if (e == -1 && pos != sprites[0].equippedArm)
		{
			shouldDisp = 0;
		}
		else
			sprites[0].equippedArm = e;

		if (e != -1)  strength = inventory[sprites[0].equippedArm].strength;

		if (strength == 2) strength = 8;
		else if (strength == 8) strength = 2;
	}
	if (inventory[pos].type == ACCESSORY)
		if ( !(e == -1 && pos != sprites[0].equippedAcc) )
			sprites[0].equippedAcc = e;

	prevStrength = strength - prevStrength;

	
	if (prevStrength != 0 || (inventory[pos].type != -1 && (inventory[pos].id == CHBLADE || inventory[pos].id == DARMOR)))
	{
		if (prevStrength < 0)
		{
			strcat(buffer, "DECREASED BY ");
			vitoa(-prevStrength, numbuffer, 2, ' ');
			strcat(buffer, numbuffer);
			strcat(buffer, ".");
		}
		else
		{
			strcat(buffer, "INCREASED BY ");
			vitoa(prevStrength, numbuffer, 2, ' ');
			strcat(buffer, numbuffer);
			strcat(buffer, ".");
		}

		if (inventory[pos].type != -1 && (inventory[pos].id == CHBLADE || inventory[pos].id == DARMOR))
			strcat(buffer, "THE ITEM IS CURSED.");

		if (shouldDisp) displayMessage(buffer, resources);
	}

	/** NOTE: SPECTACLES REMOVED
	//If you are equipping/unequipping the spectacles, reveal/hide things as necessary
	//Reveal
	if ( (prev == -1 || inventory[prev].id != SPECTACLES) && (sprites[0].equippedAcc != -1 && inventory[sprites[0].equippedAcc].id == SPECTACLES))
	{
		uint8_t x, y;
		for (x = 0; x < MAP_SIZE_X; x++)
			for (y = 0; y < MAP_SIZE_Y;  y++)
				revealTrap(x, y, resources, 0);

		for (x = 0; x < MAX_SPRITES; x++)
			if (sprites[x].isAlive && sprites[x].isVisible && sprites[x].id == MIMIC)
				sprites[x].flag3 = 1;
	}

	//Hide
	if ( (prev != -1 && inventory[prev].id == SPECTACLES) && (sprites[0].equippedAcc == -1 || inventory[sprites[0].equippedAcc].id != SPECTACLES))
	{
		uint8_t x, y;
		for (x = 0; x < MAP_SIZE_X; x++)
			for (y = 0; y < MAP_SIZE_Y; y++)
				if (map[y*MAP_SIZE_X+x]==TRAP_POISON || map[y*MAP_SIZE_X+x]==TRAP_HEALTH || map[y*MAP_SIZE_X+x]==TRAP_BOMB || map[y*MAP_SIZE_X+x]==TRAP_CONFUSION)
				{
					setActualTile2(x * 2, y * 2, floor1, floor2, floor2, floor1);
				}

		for (x = 0; x < MAX_SPRITES; x++)
			if (sprites[x].isAlive && sprites[x].isVisible && sprites[x].id == MIMIC)
			{
				sprites[x].flag3 = 0;
				sprites[x].frame = 0;
			}
	}
	*/
}
void makeDecision(uint8_t sx, uint8_t sy, uint8_t* resources[])
{
	uint8_t offx = width/2 - INVENTORY_WIDTH/2;
	uint8_t offy = height/2 - INVENTORY_HEIGHT/2;

	while (1)
	{
		uint8_t t = inventory[sy*4+sx].type;

		if (keyPress('1'))
		{
			if ( t == ITEM )
			{
				useConsumable( inventory[sy*4+sx].name, resources );
				inventory[sy*4+sx].type = -1;
				break;
			}
			else if ( t == LTREASURE )
			{
				inventory[sy*4+sx].type = -1;
				break;
			}
			else
			{
				if (sprites[0].equippedAcc == sy * 4 + sx ||
					sprites[0].equippedArm == sy * 4 + sx ||
					sprites[0].equippedWep == sy * 4 + sx )
					changeEquipment(sy*4+sx, -1, resources);
				else
					changeEquipment(sy * 4 + sx, sy * 4 + sx, resources);

				break;
			}
		}
		if (keyPress('2'))
		{
			if ( t == LTREASURE )
				break;
			else
			{
				changeEquipment(sy*4+sx, -1, resources);
				inventory[sy*4+sx].type = -1;

				break;
			}
		}
		if (keyPress('3')) 
		{
			if ( t == WEAPON || t == ARMOR || t == ACCESSORY || t == ITEM)
				break;

		}

		vClearScreen(0);
		vDrawObject(offx, offy, resources[INVENTORY]);
		vDrawObject(offx+INV_START_X + sx * INV_SEP_X,
					offy+INV_START_Y + sy * INV_SEP_Y,
					resources[INVPICK]);

		uint8_t x, y;
		for (x = 0; x < 4; x++)
			for (y = 0; y < 2; y++)
				if (inventory[y*4+x].type != -1)
				{
					vDrawObject(	offx+INV_TLCORNERX + x * INV_SEP_X,
									offy+INV_TLCORNERY + y * INV_SEP_Y,
									resources[ inventory[ y * 4 + x ].id ] );

					if (sprites[0].equippedAcc == y * 4 + x ||
						sprites[0].equippedWep == y * 4 + x ||
						sprites[0].equippedArm == y * 4 + x)
							vPrint(	MODE_TRANS,
									offx+INV_TLCORNERX + MOONROCK_WIDTH + x * INV_SEP_X - THEFONT_WIDTH,
									offy+INV_TLCORNERY + y * INV_SEP_Y,
									"E" );
				}

		
		if (t == WEAPON || t == ARMOR || t == ACCESSORY)
		{
			if (sprites[0].equippedWep == sy * 4 + sx ||
				sprites[0].equippedArm == sy * 4 + sx ||
				sprites[0].equippedAcc == sy * 4 + sx)
				vPrint( MODE_TRANS, offx, offy+INV_TEXT_Y, "1.UNEQUIP 2.DROP 3.BACK" );
			else
				vPrint( MODE_TRANS, offx, offy+INV_TEXT_Y, "1.EQUIP 2.DROP 3.BACK" );
		}
		else if (t != LTREASURE)
			vPrint( MODE_TRANS, offx, offy+INV_TEXT_Y, "1.USE 2.DROP 3.BACK" );
		else
			vPrint( MODE_TRANS, offx, offy+INV_TEXT_Y, "1.DROP 2.BACK");

		vFlipScreen(1);
	}
}

void inv()
{
	uint8_t* resources[NUM_RESOURCES];
	initResources(resources, NUM_RESOURCES);

	LoadPackedRes(INVENTORY, INVENTORY_UNPACKED_SIZE, &resources[INVENTORY]);
	LoadPackedRes(INVPICK, INVPICK_UNPACKED_SIZE, &resources[INVPICK]);
	LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);

	uint8_t i;
	for (i = 0; i < MAX_INVENTORY; i++)
		if (inventory[i].type != -1)
			if (resources[inventory[i].id] == NULL)
				LoadPackedRes(inventory[i].id, MOONROCK_UNPACKED_SIZE, &resources[inventory[i].id]);

	uint8_t sx = 0, 
			sy = 0;

	uint8_t offx = width/2 - INVENTORY_WIDTH/2;
	uint8_t offy = height/2 - INVENTORY_HEIGHT/2;

	while (1)
	{
		if (keyPress('*'))
			break;
		
		if (buttonPress(KEY_RIGHT)	&& sx < 3)	sx++;
		if (buttonPress(KEY_LEFT)	&& sx > 0)	sx--;
		if (buttonPress(KEY_UP)		&& sy > 0)		sy--;
		if (buttonPress(KEY_DOWN)	&& sy < 1)	sy++;
		if (keyPress('5') && inventory[sy*4+sx].type != -1)
		{
			makeDecision(sx, sy, resources);
		}

		vClearScreen(0);
		vDrawObject(offx, offy, resources[INVENTORY]);
		vDrawObject(offx+INV_START_X + sx * INV_SEP_X,
					offy+INV_START_Y + sy * INV_SEP_Y,
					resources[INVPICK]);

		uint8_t x, y;
		for (x = 0; x < 4; x++)
			for (y = 0; y < 2; y++)
				if (inventory[y*4+x].type != -1)
				{
					vDrawObject(	offx+INV_TLCORNERX + x * INV_SEP_X,
									offy+INV_TLCORNERY + y * INV_SEP_Y,
									resources[ inventory[ y * 4 + x ].id ] );

					if (sprites[0].equippedAcc == y * 4 + x ||
						sprites[0].equippedWep == y * 4 + x ||
						sprites[0].equippedArm == y * 4 + x)
							vPrint(	MODE_TRANS,
									offx+INV_TLCORNERX + MOONROCK_WIDTH + x * INV_SEP_X - THEFONT_WIDTH,
									offy+INV_TLCORNERY + y * INV_SEP_Y,
									"E" );
										
				}

		if (inventory[sy*4+sx].type != -1)
			vPrint( MODE_TRANS, offx, offy+INV_TEXT_Y,
					inventory[ sy * 4 + sx ].name );

		vPrint(MODE_TRANS, offx+INVENTORY_WIDTH-THEFONT_WIDTH*8, offy+INV_TEXT_Y, "*. BACK");

		vFlipScreen(1);

		if (demo && vGetTickCount() - demoTime > 1000 * 60 * DEMO_TIME) break;
	}

	freeAllResources(resources, NUM_RESOURCES);
}
