import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import com.nokia.mid.ui.*;

public class Sprite
{
	private Image[] imageFrames;
	
	public int x, y;
	public int width, height;
	public int speed;
	
	public int numFrames;
	
	//Sprite animation
	public int currFrame;
	public int startFrame, endFrame;
	public long timeDelay, timeStart;
	
	//Build a new sprite from a file.  This sprite should not
	//be drawn directly, but instead should act as a "template"
	//sprite for future sprites that look and act the same
	Sprite(String fileName, int width)
	{
		this.width = width;
				
		//Break apart the image into a bunch of different frames
		try
		{
			Image imageTotal = Image.createImage(fileName);
			this.height = imageTotal.getHeight();
				
			imageFrames = new Image[imageTotal.getWidth()/width];
			numFrames = imageFrames.length;
			
			for (int i = 0; i < imageFrames.length; i++)
			{
				imageFrames[i] = DirectUtils.createImage(width, height, 0);
				DirectGraphics dg = DirectUtils.getDirectGraphics(imageFrames[i].getGraphics());	
				dg.drawImage(imageTotal, -i * width, 0, Graphics.TOP | Graphics.LEFT, 0);
			}
			
		}
		catch (Exception e)
		{
			System.out.println("Error loading sprite: " + e.getMessage());
		}	
	}
	
	Sprite(String fileName)
	{
		try
		{
			imageFrames = new Image[1];
			imageFrames[0] = Image.createImage(fileName);
			this.height = imageFrames[0].getHeight();
			this.width = imageFrames[0].getWidth();
		}
		catch (Exception e)
		{
			System.out.println("Error loading sprite: " + e.getMessage());
		}
	}
	
	//Create a sprite from another sprite.
	Sprite(Sprite template)
	{
		this.imageFrames 	= template.imageFrames;
		this.width 			= template.width;
		this.height			= template.height;
		this.numFrames		= template.numFrames;
		this.currFrame		= 0;
		this.timeStart		= System.currentTimeMillis();
		this.timeDelay		= 0;
		this.startFrame		= 0;
		this.endFrame		= 0;
		
	}
	
	public void animate()
	{
		long time = System.currentTimeMillis();
		if (time - timeStart >= timeDelay)
		{
			timeStart = time;
			currFrame++;
			
			if (currFrame > endFrame)
				currFrame = startFrame;
		}
	}
	
	public void paint(Graphics g)
	{
		DirectGraphics dg = DirectUtils.getDirectGraphics(g);
		dg.drawImage(imageFrames[currFrame], x, y, Graphics.TOP | Graphics.LEFT, 0);
	}
	
	public boolean isTemplate(Sprite template)
	{
		return (imageFrames == template.imageFrames &&
				width == template.width &&
				height == template.height &&
				numFrames == template.numFrames);
	}
	
}
