#include <vmgp.h>
#include <datacertlib.h>

#include "res.h"
#include "utility.h"
#include "rmg.h"
#include "app.h"
#include "globals.h"

#include "introduction.h"
#include "title.h"
#include "menu.h"

#define MPC_FILENAME "DungeonKid.mpc"

// *******************************************************
// CHECK FOR BOTH EMBEDDED AND EXTERNAL CERTIFICATES
// NOTE: Check that you have enough reserved memory in 
//       dataheap for data certificate
// *******************************************************
static int32_t CheckCert(void)
{
  int32_t ret, ret2;

  // Check External DataCert
  ret = vlDataCertCheckFile(MPC_FILENAME);

  // Check Internal DataCert
  if (ret != DATACERT_SUCCESS)
  {
    ret2 = ret;
    ret = vlDataCertCheckResource(DATACERT);
    if (ret == DATACERT_FAILURE)
      ret = ret2;
  }

  return ret;
}

int main()
{
	int ret;
	ret = CheckCert();
	if (ret == DATACERT_FAILURE)
		demo = 1;    
	else if (ret == DATACERT_EXPIRED)  
	{
		vMsgBox(VMB_SMALL, "Game Expired");
		vTerminateVMGP();    
	}

	loadCollection();
	saveCollection();
	getSpecs(&width, &height, &canVibrate);

	vSetTransferMode(MODE_TRANS);
	
	uint8_t* resFont;
	LoadPackedRes(THEFONT, THEFONT_UNPACKED_SIZE, &resFont);
	VMGPFONT font =
	{ 
		resFont + THEFONT_CHARTBLSIZE,
		resFont,
		THEFONT_BPP,
		THEFONT_WIDTH,
		THEFONT_HEIGHT,
		THEFONT_PALINDEX	
	};
	vSetActiveFont(&font);
	vSetForeColor(255);
	
	loadSettings();

	introduction();
	title();

	menu();
	
	vDisposePtr(resFont);
	vTerminateVMGP();
	return 0;
}
