#include "collection.h"

#include <vmgp.h>
#include "globals.h"
#include "res.h"
#include "app.h"
#include "utility.h"

#include "about.h"
#include "help.h"
#include "collection.h"
#include "game.h"

void menu()
{
	uint8_t* resources[NUM_RESOURCES];
	initResources(resources, NUM_RESOURCES);
	
	LoadPackedRes(MENUBOX, MENUBOX_UNPACKED_SIZE, &resources[MENUBOX]);
	LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);

	if (sound)
	{
		LoadPackedRes(MMENU, MMENU_UNPACKED_SIZE, &music);
		vPlayResource(music, MMENU_UNPACKED_SIZE, SOUND_TYPE_MIDI );
	}

	demoTime = vGetTickCount();

	while(1)
	{
		debug = 1;

		//Process the input
		if (keyPress('1'))
		{
			freeAllResources(resources, NUM_RESOURCES);
			game(0);
			LoadPackedRes(MENUBOX, MENUBOX_UNPACKED_SIZE, &resources[MENUBOX]);
			LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);
			if (sound)
			{
				vDisposePtr(music);
				LoadPackedRes(MMENU, MMENU_UNPACKED_SIZE, &music);
				vPlayResource(music, MMENU_UNPACKED_SIZE, SOUND_TYPE_MIDI );
			}
		}
		if (keyPress('2') && saveGameExists())
		{
			freeAllResources(resources, NUM_RESOURCES);
			game(1);
			LoadPackedRes(MENUBOX, MENUBOX_UNPACKED_SIZE, &resources[MENUBOX]);
			LoadPackedRes(TEXTBOX, TEXTBOX_UNPACKED_SIZE, &resources[TEXTBOX]);
			if (sound)
			{
				vDisposePtr(music);
				LoadPackedRes(MMENU, MMENU_UNPACKED_SIZE, &music);
				vPlayResource(music, MMENU_UNPACKED_SIZE, SOUND_TYPE_MIDI);
			}
		}
		if (keyPress('3'))
			collection();
		if (keyPress('4'))
			//vMsgBox(VMB_TITLE, HELP, "HELP");
			vMsgBox(VMB_BIG, HELP);
		if (keyPress('5'))
			about();
		if (keyPress('6'))
		{
			sound = !sound;
			if (!sound)
			{
				vPlayResource(music, MMENU_UNPACKED_SIZE, SOUND_TYPE_MIDI | SOUND_FLAG_STOP);
				vDisposePtr(music);
				music = NULL;
			}
			else
			{
				LoadPackedRes(MMENU, MMENU_UNPACKED_SIZE, &music);
				vPlayResource(music, MMENU_UNPACKED_SIZE, SOUND_TYPE_MIDI);
			}

			saveSettings();
		}
		if (keyPress('7'))
		{
			doesVibrate = !doesVibrate;
			saveSettings();
		}
		if (keyPress('8'))
		{
			if (prompt("ARE YOU SURE YOU WANT TO QUIT", resources) == 1)
				break;
		}
		
		//Draw the menu
		vClearScreen(0);

		vDrawObject(width/2 - MENUBOX_WIDTH/2, height/2 - MENUBOX_HEIGHT/2, resources[MENUBOX]);
		uint8_t textX = width/2 - MENUBOX_WIDTH/2 + MENUBOX_START_TEXT_X;
		uint8_t textY = height/2 - MENUBOX_HEIGHT/2 + MENUBOX_START_TEXT_Y;

		vPrint(MODE_TRANS, textX, textY, "1. START");		textY += THEFONT_HEIGHT;
		if (saveGameExists()) vPrint(MODE_TRANS, textX, textY, "2. CONTINUE");	textY += THEFONT_HEIGHT;
		vPrint(MODE_TRANS, textX, textY, "3. COLLECTION");	textY += THEFONT_HEIGHT;
		vPrint(MODE_TRANS, textX, textY, "4. HELP");		textY += THEFONT_HEIGHT;
		vPrint(MODE_TRANS, textX, textY, "5. ABOUT");		textY += THEFONT_HEIGHT;
		vPrint(MODE_TRANS, textX, textY, "6. SOUND");
		vSetForeColor(vRGB(255,0,0));

		if (sound)
			vPrint(MODE_TRANS, textX + THEFONT_WIDTH * 9, textY, "ON"); 
		else
			vPrint(MODE_TRANS, textX + THEFONT_WIDTH * 9, textY, "OFF");
		
		textY += THEFONT_HEIGHT;

		vSetForeColor(255);
		vPrint(MODE_TRANS, textX, textY, "7. VIBRATION");
		vSetForeColor(vRGB(255,0,0));

		if (doesVibrate)
			vPrint(MODE_TRANS, textX + THEFONT_WIDTH * 13, textY, "ON"); 
		else
			vPrint(MODE_TRANS, textX + THEFONT_WIDTH * 13, textY, "OFF");
		
		textY += THEFONT_HEIGHT;

		if (demo)
		{
			vSetForeColor(vRGB(255,0,0));
			vPrint(MODE_TRANS, width/2-(THEFONT_WIDTH*4)/2, 0, "DEMO");
		}
		vSetForeColor(255);
		vPrint(MODE_TRANS, textX, textY, "8. QUIT");
		
		vFlipScreen(1);

		if (demo && vGetTickCount() - demoTime > 1000 * 60 * DEMO_TIME) break;
	}

	freeAllResources(resources, NUM_RESOURCES);

	if (sound)
	{
		vDisposePtr(music);
	}

	if (demo)
	{
		vMsgBox(VMB_INFO, "Thank you for playing the demo version of Dungeon Kid. Please purchase the full version to stop the time limit.");
	}
}
