#include <vmgp.h>
#include <vstream.h>
#include <datacertlib.h>
#include "numberdrop.h"
#include "font.h"
#include "game.h"

#define MPC_FILENAME "numberdrop.mpc"

uint8_t sound;
uint8_t demomode;
uint8_t spanish;
uint8_t *m1,*m2,*m3,*m4;

void loadmenu()
{
	LoadPackedRes(MENU1,MENU1_UNPACKED_SIZE,&m1);
	LoadPackedRes(MENU2,MENU2_UNPACKED_SIZE,&m2);
	LoadPackedRes(MENU3,MENU3_UNPACKED_SIZE,&m3);
	LoadPackedRes(MENU4,MENU4_UNPACKED_SIZE,&m4);
}

void drawborder()
{
	vClearScreen(VMGP_BLACK);
	vDrawObject(0,0,m1);
	vDrawObject(0,60,m2);
	vDrawObject(SCREEN_WIDTH-27,60,m3);
	vDrawObject(0,169,m4);
}

void killmenu()
{
	vDisposePtr(m1);
	vDisposePtr(m2);
	vDisposePtr(m3);
	vDisposePtr(m4);
}

void splash()
{
	uint8_t	*logo;
	uint8_t gx;

	LoadPackedRes(LOGO,LOGO_UNPACKED_SIZE ,&logo);

	for(gx=SCREEN_WIDTH-20;gx>(SCREEN_WIDTH/2)-16;gx--){
		vClearScreen(vRGB(255,255,255));
		vDrawObject((SCREEN_WIDTH/2)-50,(SCREEN_HEIGHT/2)-40,logo);
		if(gx%5<3)
			vDrawObject(gx,60,&GHOST1);
		else
			vDrawObject(gx,60,&GHOST2);
		delay(4);
	}
	delay(45);
	vDisposePtr(logo);
}

void titleScreen()
{
	uint8_t	*t1,*t2;	
	uint8_t c;

	LoadPackedRes(TITLE1,TITLE1_UNPACKED_SIZE ,&t1);
	LoadPackedRes(TITLE2,TITLE2_UNPACKED_SIZE ,&t2);

	vClearScreen(VMGP_BLACK);
	vDrawObject(0,0,t1);
	vDrawObject(0,104,t2);
	vFlipScreen(1);
	
	delay(400);

	vDisposePtr(t1);
	vDisposePtr(t2);
}

void highScore()
{
	displayHiScores(1,m1,m2,m3,m4);
	waitForButton();
}

void credits()
{
	uint8_t sx=40,sy=55,yinc=12;

	drawborder();
	printMain(sx+25,sy+yinc,"CREDITS",VMGP_WHITE);
	printMain(sx,sy+yinc*2,"DESIGN:",VMGP_RED);
	printMain(sx,sy+yinc*3,"DUSTIN HUBBARD",VMGP_WHITE);
	printMain(sx,sy+yinc*4,"PROGRAMMING:",VMGP_RED);
	printMain(sx,sy+yinc*5,"AUSTIN RUSSELL",VMGP_WHITE);
	printMain(sx,sy+yinc*6,"PRODUCER:",VMGP_RED);
	printMain(sx,sy+yinc*7,"JONATHAN DENMARK",VMGP_WHITE);

	vFlipScreen(1);
	while(vScanKeys()){}
	waitForButton();
}

void drawMainMenu(uint8_t lvl)
{
	uint8_t levelst[2];
	vitoa(lvl, levelst, 2, '0'); 
	uint8_t sx=40,sy=55,xinc=14,yinc=12,x2=50;

	vClearScreen(VMGP_BLACK);
	drawborder();
	
	if(demomode==1){
		printMain(sx+75,sy+yinc*8,"DEMO",VMGP_RED);
	}
	if(spanish==NO){
		printMain(sx,sy+yinc,"1.",VMGP_RED);
		printMain(sx+xinc,sy+yinc,"START",VMGP_WHITE);
		printMain(sx,sy+yinc*2,"2.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*2,"LEVEL",VMGP_WHITE);
			printMain(sx+x2,sy+yinc*2,levelst,VMGP_RED);
		printMain(sx,sy+yinc*3,"3.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*3,"HIGH SCORES",VMGP_WHITE);
		printMain(sx,sy+yinc*4,"4.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*4,"CREDITS",VMGP_WHITE);
		printMain(sx,sy+yinc*5,"5.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*5,"SOUND",VMGP_WHITE);
		if(sound==ON)
			printMain(sx+x2,sy+yinc*5,"ON",VMGP_RED);
		else
			printMain(sx+x2,sy+yinc*5,"OFF",VMGP_RED);
		printMain(sx,sy+yinc*6,"6.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*6,"ESPANOL",VMGP_WHITE);
		printMain(sx,sy+yinc*7,"7.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*7,"HELP",VMGP_WHITE);
		printMain(sx,sy+yinc*8,"8.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*8,"EXIT",VMGP_WHITE);
	}
	else
	{
		printMain(sx,sy+yinc,"1.",VMGP_RED);
		printMain(sx+xinc,sy+yinc,"COMIENZO",VMGP_WHITE);
		printMain(sx,sy+yinc*2,"2.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*2,"NIVEL",VMGP_WHITE);
			printMain(sx+x2,sy+yinc*2,levelst,VMGP_RED);
		printMain(sx,sy+yinc*3,"3.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*3,"ALTOS PUNTAJES",VMGP_WHITE);
		printMain(sx,sy+yinc*4,"4.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*4,"CREDITOS",VMGP_WHITE);
		printMain(sx,sy+yinc*5,"5.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*5,"SONIDOS",VMGP_WHITE);
		if(sound==ON)
			printMain(sx+60,sy+yinc*5,"EN",VMGP_RED);
		else
			printMain(sx+60,sy+yinc*5,"DE",VMGP_RED);
		printMain(sx,sy+yinc*6,"6.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*6,"ENGLISH",VMGP_WHITE);
		printMain(sx,sy+yinc*7,"7.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*7,"AYUDA",VMGP_WHITE);
		printMain(sx,sy+yinc*8,"8.",VMGP_RED);
		printMain(sx+xinc,sy+yinc*8,"SALIDA",VMGP_WHITE);
	}
	vFlipScreen(1);
}

uint8_t mainMenu()
{	
	int8_t lvl=1;
	uint8_t keys;

	while(vScanKeys()){}
	drawMainMenu(lvl);
	delay(60);

	while(1)
	{
		drawMainMenu(lvl);
		keys=vScanKeys();

		switch(keys)
		{
		case '1':
			killmenu();
			if(demomode==0){
				while(vScanKeys()){}
				game(lvl,sound,NO);
				loadmenu();
				return 0;
			}
			else{
				while(vScanKeys()){}
				game(lvl,sound,YES);
				loadmenu();
				demomode=1;
				return 0;
			}
		case '2':
			if(demomode==0){
				if(lvl==15) lvl=1;
				else lvl++;
				break;
			}
			else{
				if(lvl==5) lvl=1;
				else lvl++;
				break;
			}
				
		case '3': highScore(); break;
		case '4': credits(); break;
		case '5': if(sound==ON) sound=OFF; else sound=ON; break;
		case '6': if(spanish==NO) spanish=YES; else spanish=NO; break;
		case '7': if(spanish==NO)
					  vMsgBox(VMB_BIG,"Save the citizens of mobiletopia by destroying the dropping numbers.\n\nPress the number on your phone that is falling from the sky to destroy it. If a number reaches the ground you will lose a citizen. Lose all 10 and the game is over!\n\nWhen a string of numbers fall type in the string from left to right to destroy it.\n\nEvery 5 rounds a bonus round occurs. After it, you will get 10 new people to protect.\n\nNumber rate and speed will increase the higher the level as well as longer strings.\n\nScoring is 100 points per number. Bonus rounds its 250 per number. Survive the Bonus round for a 50000 point bonus!\n\n\"For support visit www.spookytornado.com\"");
			      else
					  vMsgBox(VMB_BIG,"Salvar los ciudadanos de Mobiletopia, destruye los nmeros que estn cayendo.\n\nPrensa el nmero en su telfono que corresponda con el numero que esta cayendo del cielo para destruirlo. Si un numero toca la tierra usted perdera un ciudadano. Si pierda los diez ciudadnos, se termina el juego\n\nCuando cae una secuencia de numeros prensa la secuencia de numeros en orden de izquierda a derecha para destruirla.\n\nDespues de cada 5 niveles usted encontrara un nivel de la Prima. Despues de el, usted recibira 10 nuevas personas para proteger.\n\nLa cantidad de numeros asi como la velocidad y la frecuencia en los cuales caen aumentara como los niveles aumentan.\n\nCada numero destruido vale 100 puntos. En los niveles de la Prima los numeros valen 250 puntos. Si usted sobrevive el nivel de la Prima , usted recibira una prima de 50000 puntos!\n\nPara recibir ayuda, visita www.spookytornado.com.");
				  break;
		case '8': return 1;
		default: break;
		}
		while(vScanKeys()){}
	}
}

static int32_t doCertCheck(int32_t hRes, int32_t size)
{
  int32_t ret;
  uint8_t *ptr;

  if ((ptr = vNewPtr(size)) == NULL)
  {
    vStreamClose(hRes);    
    return DATACERT_FAILURE;  
  }
  
  vStreamRead(hRes, ptr, size);    
  vStreamClose(hRes);
  
  ret = vlDataCertCheck(ptr);

  vDisposePtr(ptr);
  
  return ret;
}

static int32_t CheckCert(void)
{
  int32_t size, ret = 0, ret2;
  int32_t hRes;  

  // Check External DataCert
  if ((hRes = vStreamOpen(MPC_FILENAME, STREAM_FILE | STREAM_READ | STREAM_BINARY)) != -1)	// #define MPC_FILENAME "EelCert.mpc" 
  {
    size = vStreamSeek(hRes, 0, VSEEK_END);
    vStreamSeek(hRes, 0, VSEEK_SET);
    ret = doCertCheck(hRes, size);          
  }
  
  // Check Internal DataCert
  if (ret != DATACERT_SUCCESS)
  {    
    if ((hRes = vResOpen(NULL, DATACERT)) != -1)
    {
      size = DATACERT_SIZE;  
	    ret2 = ret;
      ret = doCertCheck(hRes, size);
	    if (ret == DATACERT_FAILURE)
	      ret = ret2;
    }
  }

  return ret;
}

int main(void)
{
	demomode=0;
	spanish=NO;
	uint8_t ret;
	ret = CheckCert();
	if (ret == DATACERT_FAILURE)
		demomode = 1;    
	else if (ret == DATACERT_EXPIRED)  
	{
		vMsgBox(VMB_SMALL, "Game Expired");
		vTerminateVMGP();    
	}
	readHiScores();
	sound=ON;
	splash();
	titleScreen();
	vClearScreen(vRGB(0,0,0));

	loadmenu();
	while(!mainMenu()){}
	killmenu();

	return 1;
}
