#include "rmg.h"

uint8_t isValid(uint8_t* map, uint8_t w, uint8_t h,
				int8_t x1, int8_t y1,
				int8_t x2, int8_t y2,
				uint8_t floor)
{
	int8_t tx, ty, ix, iy;
	
	if (x1 < x2) ix = 1; else ix = -1;
	if (y1 < y2) iy = 1; else iy = -1;
	
	int8_t s = 3;
	if (ix < 0) { x1+=s; x2-=s; } else { x1-=s; x2+=s; }
	if (iy < 0) { y1+=s; y2-=s; } else { y1-=s; y2+=s; }
	
	for (tx = x1; tx != x2; tx += ix)
		for (ty = y1; ty != y2; ty += iy)
			if (tx < 0 || ty < 0 || tx >= w || ty >= h || map[ty * w + tx] == floor)
				return 0;
		
	return 1;
} 

void inline setActualTile(uint8_t* map, uint8_t x, uint8_t y, uint8_t w,
						  uint8_t t1, uint8_t t2, uint8_t t3, uint8_t t4)
{
	map[ y * w + x ] = t1;
	map[ y * w + x + 1 ] = t2;
	map[ (y+1) * w + x ] = t3;
	map[ (y+1) * w + (x+1) ] = t4;
}

void inline setActualTile2( uint8_t x, uint8_t y,
							uint8_t t1, uint8_t t2, uint8_t t3, uint8_t t4)
{
	vMapSetTile( x,		y,		t1	);
	vMapSetTile( x+1,	y,		t2	);
	vMapSetTile( x,		y+1,	t3	);
	vMapSetTile( x+1,	y+1,	t4	);
}

uint8_t* generateRandomMap(MAP_HEADER *header, int8_t width, int8_t height, uint32_t seed, uint8_t * floor1, uint8_t * floor2)
{
	uint8_t* map = (uint8_t*)vNewPtr( width * height * sizeof(uint8_t) );
		
	//Clear the map (filling it with walls)
	memset( (void*)map, WALL, width * height * sizeof(uint8_t) );
	

	//-Generate a random map--
	//--place room in the center
	uint8_t centerX = width/2, centerY = height/2;
	uint8_t minX = centerX-2, 
			maxX = centerX+2, 
			minY = centerY-2, 
			maxY = centerY+2;

	int8_t x, y;
	for (x = centerX - 2; x <= centerX + 2; x++)
		for (y = centerY - 2; y <= centerY + 2; y++)
			map[ y * width + x ] = FLOOR;

	vSetRandom(seed);

	//The maxRooms is not necessarily the total number of rooms
	uint16_t maxRooms = 2500;
	while (maxRooms > 1)
	{
		maxRooms--;
		
		//Pick a random spot.  If it's a wall connected to a room, put a 4 * 4 room there
		uint8_t x = vGetRandom() % (width-1) + 1,
				y = vGetRandom() % (height-1) + 1,
				w = vGetRandom() % 4 + 4,
				h = vGetRandom() % 4 + 4;
		
		if (map[ y * width + x ] == WALL || map[ y * width + x ] == COLUMN)
		{	
			uint8_t d = 0;

			//Determine a random direction for the room to be built in
			int8_t dx = vGetRandom()%2, dy = vGetRandom()%2;
			if (dx == 0) dx = -1; else dx = 1;
			if (dy == 0) dy = -1; else dy = 1;
			
			//Determine if the room is close enough to another room that they
			//can be connected using a 3-tile hallway
			if (dx == 1 && x - 4 > 0 && map[ y * width + (x-4) ] == FLOOR || 
				dx == -1 && x + 4 < width && map[ y * width + (x+4) ] == FLOOR)
			{
				d = 0;
			}
			else if (dy == 1 && y - 4 > 0 && map[ (y-4) * width + x ] == FLOOR ||
					 dy == -1 && y + 4 < height && map[ (y+4) * width + x ] == FLOOR)
			{
				d = 1;
			}
			else
				continue;
			
			
			//This is called down here so that if the above conditions are false, it will
			//never be called and thus quite a number of iterations are saved.
			if (!isValid(map, width, height, x, y, x + w * dx, y + h * dy, FLOOR)) continue;
			
			//We now know we're creating a valid room.
			//Add columns & windows where appropriate
			
			
			map[ (y-1*dy) * width + x ] = COLUMN;
			map[ (y+h*dy) * width + x ] = COLUMN;
			map[ (y-1*dy) * width + x + (w-1) * dx ] = COLUMN;
			map[ (y+h*dy) * width + x + (w-1) * dx ] = COLUMN;

			if (w % 2 == 1)
			{
				map[ (y-1*dy) * width + (x + (w/2) * dx ) ] = WINDOW;
				map[ (y+h*dy) * width + (x + (w/2) * dx ) ] = WINDOW;
			}

			//These loops are done here as opposed to inside the if statements because they
			//can only be performed if isValid is true
			//These loops build a passage between rooms that is 3 tiles long
			uint8_t i;
			if (d == 0) 
				for (i = 1; i < 4; i++) map[ y * width + (x - i * dx) ] = FLOOR;
			else
				for (i = 1; i < 4; i++) map[ (y - i * dy) * width + x ] = FLOOR;

			//Build the room using floor tiles
			int8_t cx, cy;
			for (cx = x; cx != x + w * dx; cx += dx)
				for (cy = y; cy != y + h * dy; cy += dy)
					if (cx>0 && cx<width && cy>0 && cy<height)
					{
						map[ cy * width + cx ] = FLOOR;
						
						//Update the bounding box so that in the next pass only the necessary
						//tiles are touched
						if (cx < minX) minX = cx;
						if (cx > maxX) maxX = cx;
						if (cy < minY) minY = cy;
						if (cy > maxY) maxY = cy;
					}

			
			
		}
	}

	if (minX < 1) minX = 1;
	if (minY < 1) minY = 1;

	
	uint8_t* actualmap = (uint8_t*)vNewPtr( (width * 2) * (height * 2) * sizeof(uint8_t) );
	memset( (void*)actualmap, 0, (width * 2) * (height * 2) * sizeof(uint8_t) );

	//--Make the map look pretty--
	int8_t	amx = 0, 
			amy = 0, 
			amw = width * 2;
	
	uint8_t f1 = vGetRandom()%16;
	uint8_t f2 = vGetRandom()%16;
	while (f2 == f1)
		f2 = vGetRandom()%16;

	if (f1 < 4)			f1 = 33 - 10 * (f1 < 2);
	else if (f1 < 8)	f1 = 37 - 10 * (f1 < 6);
	else if (f1 < 12)	f1 = 73 - 10 * (f1 < 10);
	else				f1 = 77 - 10 * (f1 < 14);

	if (f2 < 4)			f2 = 33 - 10 * (f2 < 2);
	else if (f2 < 8)	f2 = 37 - 10 * (f2 < 6);
	else if (f2 < 12)	f2 = 73 - 10 * (f2 < 10);
	else				f2 = 77 - 10 * (f2 < 14);

	*floor1 = f1;
	*floor2 = f2;

	for (x = minX-1; x <= maxX+1; x ++)
	{
		if (x < 0 || x >= width) continue;

		amx = x << 1;
		for (y = minY-1; y <= maxY+1; y ++)
		{
			if (y < 0 || y >= height) continue;

			amy = y << 1;

			//Set down the floor tiles
			if ( map[ y * width + x ] == FLOOR )
			{
				setActualTile(actualmap, amx, amy, amw, f1, f2, f2, f1);
			}
			else
			{
				//Find out of the neighboring cells are a wall or not and store the results
				//in a boolean array
				int8_t wall[3][3], xx, yy;
				for ( xx = -1; xx < 2; xx ++)
					for ( yy = -1; yy < 2; yy++)
						wall[xx+1][yy+1] = !( x + xx < 0 || y + yy < 0 || x + xx >= width || 
											  y + yy >= height || 
											  map[(y+yy)*width+(x+xx)] == WALL || 
											  map[(y+yy)*width+(x+xx)] == COLUMN ||
											  map[(y+yy)*width+(x+xx)] == WINDOW);		 

				//Place corners and walls depending on the pattern of walls around this wall
				if ( !wall[0][0] && !wall[1][0] && !wall[2][0] && !wall[0][1] && !wall[2][1] && !wall[0][2] && !wall[1][2] && wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 1, 2, 11, 12);
				else if ( !wall[0][0] && !wall[1][0] && wall[2][0] && !wall[0][1] && !wall[2][1] && !wall[0][2] && !wall[1][2] && !wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 81, 82, 91, 92);
				else if ( !wall[0][0] && !wall[1][0] && !wall[2][0] && !wall[0][1] && !wall[2][1] && wall[0][2] && !wall[1][2] && !wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 9, 10, 19, 20);
				else if ( wall[0][0] && !wall[1][0] && !wall[2][0] && !wall[0][1] && !wall[2][1] && !wall[0][2] && !wall[1][2] && !wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 89, 90, 99, 100);
				else if ( wall[0][0] && wall[1][0] && wall[0][1] && !wall[2][1] && !wall[1][2] && !wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 1, 2, 11, 12);
				else if ( wall[1][0] && wall[2][0] && !wall[0][1] && wall[2][1] && !wall[0][2] && !wall[1][2])
					setActualTile(actualmap, amx, amy, amw, 9, 10, 19, 20);
				else if ( !wall[1][0] && !wall[2][0] && wall[0][1] && !wall[2][1] && wall[0][2] && wall[1][2])
					setActualTile(actualmap, amx, amy, amw, 81, 82, 91, 92);
				else if ( !wall[0][0] && !wall[1][0] && !wall[0][1] && wall[2][1] && wall[1][2] && wall[2][2])
					setActualTile(actualmap, amx, amy, amw, 89, 90, 99, 100);
				else if ( y + 1 < height && map[ (y+1) * width + x ] == FLOOR ||
						  y - 1 > 0	&& map[ (y-1) * width + x ] == FLOOR)
				{
					if ( map[ y * width + x ] == WALL )
						setActualTile(actualmap, amx, amy, amw, 83, 84, 93, 94);
					else if ( map[ y  * width + x ] == COLUMN )
						setActualTile(actualmap, amx, amy, amw, 3, 4, 13, 14);
					else if ( map[y * width + x ] == WINDOW )
						setActualTile(actualmap, amx, amy, amw, 7, 8, 17, 18);
				}
				else if ( x + 1 < width && map[ y * width + x + 1 ] == FLOOR ||
						  x - 1 > 0 && map[ y * width + x - 1 ] == FLOOR)
					setActualTile(actualmap, amx, amy, amw, 21, 22, 31, 32);
			}
			
		}
	}
	
	//Fill the header structure
	//We leave out the tile data, because we may not want to load the tiles until after
	//the map is generated
	header->x			= 0;
	header->y			= 0; 
	header->flag		= 0;
	header->xpan		= 0;
	header->ypan		= 0;
	header->width		= width*2;
	header->height		= height*2;
	header->mapoffset	= actualmap;

	return map;
}
